// $Id: CheckOptics.C,v 1.7 2005/10/31 14:01:03 thea Exp $
// Author: A.Thea   Dec, 28 2004

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: CheckOptics                                                          *
 *  Package: Macros                                                          *
 *  Coordinator: Alessandro.Thea                                             *
 *                                                                           *
 *****************************************************************************/

#include <vector>

#include <iostream>
#if !defined(__CINT__)
#include "ESystemOfUnits.hh"
#include "Etypes.hh"
#include "EsafRandom.hh"
#include "ParentPhoton.hh"
#include "Photon.hh"
#include "KOpticalSystem.hh"
#include "KIdealFocalSurface.hh"
#include "ParamOpticalSystem.hh"
#endif

#include "TStyle.h"
#include "TH1.h"
#include "TH2F.h"
#include "TVector3.h"
#include "TGraph.h"
#include "TMultiGraph.h"
#include "TCanvas.h"
#include "TGaxis.h"
#include "TLegend.h"
#include "TRandom.h"

#include <map>
using namespace std;
using namespace sou;
using namespace TMath;

// paramters NOT read from config files
Double_t gDiscRadius = 1500.*mm; // mm
Double_t gEusoRadius = 1250.*mm; // mm
Double_t gRmax = 30.; // mm
Double_t gTrigDiam = 5.; // mm

enum EOpticsModule { kKOptics, kParamOptics };

void CheckOptics(Int_t nph = 1000000, Double_t theta = 0, Double_t phi=0, 
                 Double_t wl=357*nm, EOpticsModule id=kKOptics);
void CheckOptics(Int_t nph = 10000, Double_t theta = 0, Double_t phi=0,
                const char* spectrum="nitro10km", EOpticsModule id=kKOptics);

void DrawPointSpreadFunction( Int_t nph = 10000, Double_t theta = 0, Double_t phi=0,
                              Double_t wl=357*nm,EOpticsModule id=kKOptics);

void FillHistograms( const TVector3* ifspos, const Int_t npos, IdealFocalSurface* ifs);
void Show();

void DoAnalysis(Int_t nph, const EsafSpectrum* spectrum, EOpticsModule id=kKOptics,
                Int_t nsteps = 31);
void DoAnalysis(Int_t nph, Double_t wl, EOpticsModule id=kKOptics,
                Int_t nsteps = 31);
void DoAnalysis(Int_t nph, const char* specname="nitro10km", EOpticsModule id=kKOptics,
                Int_t nsteps = 31);

//______________________________________________________________________________
void CheckOptics(Int_t nph, Double_t theta, Double_t phi, Double_t wl, EOpticsModule id) {
    //
    // Simulate and collect a uniform beam of photons on the chosen optical
    // system.
    //

    EsafSpectrum spectrum(wl);

    DrawPointSpreadFunction(nph,theta,phi,&spectrum,id);

    Show();
}

//______________________________________________________________________________
void CheckOptics(Int_t nph, Double_t theta, Double_t phi, const char* specname, EOpticsModule id) {

    EsafSpectrum spectrum(specname);

    DrawPointSpreadFunction(nph,theta,phi,&spectrum,id);

    Show();
}

//______________________________________________________________________________
void DrawPointSpreadFunction( Int_t nph, Double_t theta, Double_t phi, 
                              const EsafSpectrum* spec ,EOpticsModule id) {
    //
    // Simulate and collect a uniform beam of photons for the chosen optical
    // system.
    //

    Printf("\nTheta: %.2f deg", theta);

    gStyle->SetPalette(88,0);
//   gStyle->SetPalette(1);
    
    theta *= DegToRad(); 
    phi   *= DegToRad(); 

    TBenchmark b;
    

    TVector3* points = new TVector3[nph];
    
    OpticalSystem *os;
    IdealFocalSurface *ifs = new KIdealFocalSurface();
    switch ( id ) {
    case kKOptics :
        os = new KOpticalSystem();
        break;
    case kParamOptics :
        os = new ParamOpticalSystem();
        break;
    default:
        cout << "Not existent OS" << endl;
        return;
    }
    Double_t pos[3] = {0,0,0};
    Double_t spos[3] = {0,0,0};

    // parent initialization with useless values
    ParentPhoton* parent = new ParentPhoton(0, spos, pos, 0., 0., 0., 0, kFluorescence);

    Photon* ph;
    Photon* phCopy;

    TRandom* rndm = EsafRandom::Get();
    Int_t ntrans(0), nhits(0);
    map<Int_t,Int_t> fates;
    
    b.Start("Tracking");
    cout << "Tracing 0%" << flush; 

    for( Int_t i(0); i<nph; i++) {
        if ( (i+1)%(nph/100) == 0 )
            cout << "\rTracing " << (i+1)/(nph/100) << '%' << flush;
            
        // new photon position
        Double_t radius = sqrt(rndm->Rndm())*gDiscRadius;
        Double_t alpha = rndm->Rndm()*TwoPi();
        pos[0] = radius*cos(alpha);
        pos[1] = radius*sin(alpha);
        
        ph = &(parent->GetPhoton());
        phCopy = ph;

        // update the parent
        parent->Build(0, spos, pos, theta, phi, spec->GetLambda(), 0, kFluorescence);

        ph = os->Transport(ph);

        if ( fates.count(phCopy->fate) == 0 )
            fates[phCopy->fate] = 0;

        fates[phCopy->fate]++;
        
        // reject dead photons and going downward
        if (!ph || ph->dir.Z() < 0.) continue;
        ntrans++;
        
        ifs->HitPosition(ph);
        if (!ph->hitIfs) continue;
        
        points[nhits] = ph->posOnIfs;
        nhits++;
        
    }
    cout << "\rTracing done" << endl;

    b.Show("Tracking");
    Printf("Transmitted photons :                 %d",ntrans);
    Printf("Hits on the ifs :                     %d",nhits);
    FillHistograms(points, nhits, ifs);

    Float_t pflux = nph/(Pi()*gDiscRadius*gDiscRadius*Cos(theta));
    Printf("Incident flux :                       %e",pflux);
    Printf("Total efficacy :                      %e",nhits/pflux);
    TH1F* psfRproj = (TH1F*)gROOT->FindObject("psfRproj");
    Int_t bin = psfRproj->GetXaxis()->FindBin(gTrigDiam/2.);
    Printf("Triggering efficacy for d = %.2f mm : %e", gTrigDiam, psfRproj->Integral(1,bin)/pflux);
    
//    map<Int_t,Int_t>::iterator it;
//    for( it=fates.begin(); it != fates.end(); it++)
//        it.first;

    Printf("Fates");
    for(Int_t i(0); i<6; i++)
        Printf("    %d                                %d",i,fates[i]);

    
    
    TH2F* psf = (TH2F*)gROOT->FindObject("psf");
    TH2F* psfzoom = (TH2F*)gROOT->FindObject("psfzoom");
    Printf("Ratio Nnearby/Ntrans :                %f",psfzoom->Integral()/psf->GetEntries());

    delete [] points;
}

//______________________________________________________________________________
void FillHistograms( const TVector3* ifspos, const Int_t npos, IdealFocalSurface* ifs ) {
    //
    // Build histograms showed in PSFDisplay
    //

    cout << "Photons to process " << npos << endl;
    Int_t nbin = 100;
    Float_t xmin, ymin, zmin;
    Float_t xmax, ymax, zmax; 
    xmin = ymin =-gEusoRadius;
    xmax = ymax = gEusoRadius;

    TH2F* thxy = (TH2F*)gROOT->FindObject("psf");
    if (thxy) delete thxy;
    thxy = new TH2F("psf", "psf", 20*nbin, xmin, xmax, 20*nbin, ymin, ymax);

    // search for the peak
    for (Int_t i=0; i< npos; i++)
        thxy->Fill(ifspos[i].X(),ifspos[i].Y());

    Int_t bx, by, bz;
    thxy->GetMaximumBin(bx, by, bz);

    
    Double_t peakX = thxy->GetXaxis()->GetBinCenter( bx );
    Double_t peakY = thxy->GetYaxis()->GetBinCenter( by );

    TVector3 dummy(0,0,0);

    Int_t nnear(0);
    Float_t spacethreshold = 20.; // mm
    const TVector3 *nearpos[npos];
    
    xmin = peakX-spacethreshold;
    ymin = peakY-spacethreshold;
    xmax = peakX+spacethreshold;
    ymax = peakY+spacethreshold;

    TH2F* htemp = (TH2F*)gROOT->FindObject("psftemp");
    if (htemp) delete htemp;
    htemp = new TH2F("psftemp", "psftemp", nbin, xmin, xmax, nbin, ymin, ymax);

    // first pass: fill the histogram
    for (Int_t i=0; i< npos; i++ ) {
        Double_t dist = Sqrt((ifspos[i].X()-peakX)*(ifspos[i].X()-peakX)
                +(ifspos[i].Y()-peakY)*(ifspos[i].Y()-peakY)); 

        if ( dist > spacethreshold) continue;
        htemp->Fill(ifspos[i].X(), ifspos[i].Y());

        nearpos[nnear] = &(ifspos[i]);
        nnear++;
    }

    cout << nnear << " photons nearby" << endl;

    //tmp
    Double_t xAv(0);
    Double_t yAv(0);
    Double_t zAv(0);
    Stat_t sum(0);


    // second scan, weighted average
    for( Int_t i(1); i<=htemp->GetNbinsX(); i++)
        for ( Int_t j(1); j<=htemp->GetNbinsY(); j++) {
            Stat_t weight = htemp->GetBinContent(i,j)*
                htemp->GetBinContent(i,j)*
                htemp->GetBinContent(i,j);

            xAv += weight*htemp->GetXaxis()->GetBinCenter(i);
            xAv += weight*htemp->GetYaxis()->GetBinCenter(j);

            sum += weight;
        }

    xAv /= sum;
    yAv /= sum;

    zAv = ifs->Profile(Sqrt(xAv*xAv+yAv*yAv));
    
    /*
    // find the center of mass around the peak
    for (Int_t i=0; i< npos; i++ ){
        Double_t dist = Sqrt((ifspos[i].X()-peakX)*(ifspos[i].X()-peakX)
                                   +(ifspos[i].Y()-peakY)*(ifspos[i].Y()-peakY)); 
        if ( dist > spacethreshold) continue;

        dummy += ifspos[i];

        nearpos[nnear] = &(ifspos[i]);
        nnear++;
    }

    cout << nnear << " photons nearby" << endl;
    if (nnear == 0) 
        return;
    
    dummy *= 1./nnear;
*/


    cout << xAv << "    "  << yAv <<  endl;

    Double_t fZoomSide=15;
    TH1F *thr = (TH1F*)gROOT->FindObject("psfRproj");
    if ( thr ) delete thr;
    thr = new TH1F("psfRproj", "Psf R-projection", nbin, 0, fZoomSide); 

    xmin = xAv-fZoomSide;
    ymin = yAv-fZoomSide;
    zmin = zAv-10.;
    xmax = xAv+fZoomSide;
    ymax = yAv+fZoomSide;
    zmax = zAv+10.;

    TH1F *thx = (TH1F*)gROOT->FindObject("psfXproj");
    if ( thx ) delete thx;
    thx = new TH1F("psfXproj", "Psf X-projection", nbin, xmin, xmax);

    TH1F *thy = (TH1F*)gROOT->FindObject("psfYproj");
    if ( thy ) delete thy;
    thy = new TH1F("psfYproj", "Psf Y-projection", nbin, ymin, ymax);

    TH1F *thz = (TH1F*)gROOT->FindObject("psfZproj");
    if ( thz ) delete thz;
    thz = new TH1F("psfZproj", "Psf Z-projection", nbin, zmin, zmax);
    
    TH2F* thxyzoom = (TH2F*)gROOT->FindObject("psfzoom");
    if (thxyzoom) delete thxyzoom;
    thxyzoom = new TH2F("psfzoom", "Psf - detail", nbin, xmin, xmax, nbin, ymin, ymax);


    Float_t x_dist, y_dist, z_dist, r;
    for (Int_t i=0; i< nnear; i++){

        thx->Fill(nearpos[i]->X());
        thy->Fill(nearpos[i]->Y());
        thz->Fill(nearpos[i]->Z());
        thxyzoom->Fill(nearpos[i]->X(),nearpos[i]->Y());

        x_dist = nearpos[i]->X()-xAv;
        y_dist = nearpos[i]->Y()-yAv;
        z_dist = nearpos[i]->Z()-yAv;
        r = Sqrt((nearpos[i]->X()-xAv)*(nearpos[i]->X()-xAv)+
                (nearpos[i]->Y()-yAv)*(nearpos[i]->Y()-yAv)+
                (nearpos[i]->Z()-zAv)*(nearpos[i]->Z()-zAv));
        if( r < gRmax) 
            thr->Fill( r );
    }

    thx->GetXaxis()->SetTitle("mm");
    thy->GetXaxis()->SetTitle("mm");

    thxy->GetXaxis()->SetTitle("mm");
    thxy->GetYaxis()->SetTitle("mm");
    thxy->GetYaxis()->SetTitleOffset(1.2);

    thxyzoom->GetXaxis()->SetTitle("mm");
    thxyzoom->GetYaxis()->SetTitleOffset(1.2);
    thxyzoom->GetYaxis()->SetTitle("mm");
    thxyzoom->GetYaxis()->SetTitleOffset(1.2);

    thr->GetXaxis()->SetTitle("mm");


    //create hint filled with the bins integral of histo
    Double_t Rintegral = thr->Integral();
    Int_t nRbins = thr->GetNbinsX();
    Double_t xRmin = thr->GetXaxis()->GetXmin();
    Double_t xRmax = thr->GetXaxis()->GetXmax();

    TH1F *thRint = (TH1F*)gROOT->FindObject("psfRint");
    if ( thRint ) delete thRint;
    thRint = new TH1F("psfRint","histo integral",nRbins,xRmin,xRmax);

    Double_t Rsum(0);
    for (Int_t j=1; j <= nRbins; j++) {
        Rsum += thr->GetBinContent(j);
        thRint->SetBinContent(j,Rsum/Rintegral);
    }
}

//______________________________________________________________________________
void Show() {
    //
    // Show PSFDisplay which contains:
    //
    // left upper  - X projection of the psf
    // left lower  - Y projection of the psf
    // center      - XY Histogram of the psf, zoom on the centroid
    // right upper - XY Histogram of the photons on the FS
    // right lower - radial distribution of photons centered on the psf
    //               and its integral
    //

    TH1F* thx = (TH1F*)gROOT->FindObject("psfXproj");
    if ( !thx ) return;
    TH1F* thy = (TH1F*)gROOT->FindObject("psfYproj");
    if ( !thy ) return;
    TH2F* thxyzoom = (TH2F*)gROOT->FindObject("psfzoom");
    if ( !thxyzoom ) return;
    TH1F* thr = (TH1F*)gROOT->FindObject("psfRproj");
    TH1F* thRint = (TH1F*)gROOT->FindObject("psfRint");
    if ( !(thr && thRint) ) return;
    TH2F* thxy = (TH2F*)gROOT->FindObject("psf");
    if ( !thxy ) return; 


    TCanvas* c1 = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("PSFDisplay");

    if ( !c1 ) c1 = new TCanvas("PSFDisplay", "Point Spread Function Display");
    c1->Clear();

    TString title = TString("Point spread function");
    Double_t lambda = 0;
    if ( lambda != 0 ) {
        TString lambdatitle = TString(", lambda =")+Form("%.1f", lambda)+" nm";
        title+=lambdatitle;
    }  
    c1->SetTitle("PSFDisplay");

    if (!c1->GetShowEventStatus())
        c1->ToggleEventStatus();

    TPad *leftup = new TPad("leftup","leftup",0.0,0.5,0.25,1.);
    TPad *leftdown = new TPad("leftdown","leftdown",0.,0.,0.25,0.5);
    TPad *center = new TPad("center","center",0.25,0.,0.75,1.);
    TPad *rightup = new TPad("rightup","rightup",0.75,0.5,1.,1);
    TPad *rightdown = new TPad("rightdown","rightdown",0.75,0.,1.,0.5);

    leftup->Draw();
    leftdown->Draw();
    center->Draw();
    rightup->Draw();
    rightdown->Draw();

    leftup->cd();
    thx->Draw();

    leftdown->cd();
    thy->Draw();

    center->cd();
    thxyzoom->SetFillColor(38);
    thxyzoom->Draw("colz");
    Double_t xc, yc;
    xc = (thxyzoom->GetXaxis()->GetXmax()+thxyzoom->GetXaxis()->GetXmin())/2.;
    yc = (thxyzoom->GetYaxis()->GetXmax()+thxyzoom->GetYaxis()->GetXmin())/2.;
    TEllipse* el = new TEllipse(xc, yc, gTrigDiam/2.);
    el->SetLineWidth(2.);
    el->SetLineColor(kRed);
    el->Draw();

    rightup->cd();
    thxy->Draw("col");

    rightdown->cd();
    thr->Draw();
    rightdown->Update();

    Float_t Rrightmax = 1.1*thRint->GetMaximum();
    Float_t Rscale = rightdown->GetUymax()/Rrightmax;
    thRint->SetLineColor(kRed);
    thRint->Scale(Rscale);
    thRint->Draw("same");

    TGaxis *axis = new TGaxis(rightdown->GetUxmax(),rightdown->GetUymin(),
            gPad->GetUxmax(), rightdown->GetUymax(),0,Rrightmax,510,"+L");
    axis->SetLineColor(kRed);
    axis->SetTextColor(kRed);
    axis->Draw();

    c1->Update();
}

//______________________________________________________________________________
void DoAnalysis(Int_t nph, const char* specname, EOpticsModule id, Int_t nsteps) {
    //
    // Single wl interface to DoAnalysis
    //
    EsafSpectrum spectrum(specname);
    DoAnalysis(nph, &spectrum, id, nsteps);

}

//______________________________________________________________________________
void DoAnalysis(Int_t nph, Double_t wl, EOpticsModule id, Int_t nsteps) {
    //
    // Spectrum name interface for DoAnalysis
    //
    TBenchmark gB;
    TString jobname = Form("BuildEfficacyTables");
    gB.Start(jobname);
    EsafSpectrum spectrum(wl);
    DoAnalysis(nph, &spectrum, id, nsteps);
    gB.Stop(jobname);
    gB.Show(jobname);
}

//______________________________________________________________________________
void DoAnalysis(Int_t nph, const EsafSpectrum* spectrum, EOpticsModule id,
    Int_t nsteps) {
    //
    //  Extracts several quantities from histograms created in FillHistograms
    //  from 0 to nsteps degrees. 
    //  
    //  3 Graphs are displayed:
    //  
    //  a. Total and triggering efficacy
    //  b. % of photons ending nearer that 10 mm to the centroid out of the
    //  photons reaching the focal surface
    //

    Double_t *theta     = new Double_t[nsteps];
    Double_t *totEff    = new Double_t[nsteps];
    Double_t *trigEff   = new Double_t[nsteps];
    Double_t *ratio     = new Double_t[nsteps];
    Double_t *rms       = new Double_t[nsteps];
    Double_t *rmsX      = new Double_t[nsteps];
    Double_t *rmsY      = new Double_t[nsteps];
    Double_t *fraction  = new Double_t[nsteps];
    Double_t *rmsfrac   = new Double_t[nsteps];
    
    
    for(Int_t i(0); i<nsteps; i++) {

        theta[i] = i;
        Float_t pflux = nph/(Pi()*gDiscRadius*gDiscRadius*Cos(theta[i]*DegToRad()));

        DrawPointSpreadFunction( nph, (Double_t)i, 0, spectrum, id);
    
        TH2F* psf = (TH2F*)gROOT->FindObject("psf");
        
        totEff[i]  = psf->Integral()/pflux;
        
        TH1F* psfRproj = (TH1F*)gROOT->FindObject("psfRproj");
        Int_t bin = psfRproj->GetXaxis()->FindBin(gTrigDiam/2.);

        trigEff[i] = psfRproj->Integral(1,bin)/pflux;

        TH2F* psfzoom = (TH2F*)gROOT->FindObject("psfzoom");
  
        ratio[i]   = psfzoom->Integral()/psf->GetEntries();

        rmsX[i]    = psfzoom->GetRMS(1);
        rmsY[i]    = psfzoom->GetRMS(2);
        rms[i]     = Sqrt((rmsX[i]*rmsX[i]+rmsY[i]*rmsY[i])/2.);
        
        fraction[i]  = psfRproj->Integral(1,bin)/psfRproj->Integral(); 

        rmsfrac[i]   = gTrigDiam/2.*Sqrt(1/(2*Log(1/(1-fraction[i]))));
        
    }

    TCanvas *c = new TCanvas("checkoptics","Check Optics Display");

    TVirtualPad* left = new TPad("left","left",0.005,0.005,0.495,0.995,0,0,0);
    TVirtualPad* right=new TPad("right","right",0.505,0.005,0.995,0.995,0,0,0);

    left->Draw();
    right->Draw();

    left->cd();

    // first pad: efficacies
    TGraph *gTotEff = new TGraph(nsteps,theta, totEff);
    gTotEff->SetLineColor(206); 
    gTotEff->SetLineWidth(3); 
    TGraph *gTrigEff = new TGraph(nsteps,theta, trigEff);
    gTrigEff->SetLineColor(210); 
    gTrigEff->SetLineWidth(3); 

    TMultiGraph *mg = new TMultiGraph("efficacies","Efficacies");
    mg->Add(gTotEff, "LP");
    mg->Add(gTrigEff, "LP");

    mg->Draw("A");
    
    TLegend *legend = new TLegend(0.7,0.88,0.99,0.99);
    legend->AddEntry(gTotEff,"eff_{FS}","lp");
    legend->AddEntry(gTrigEff,"eff_{trig}","lp");
    legend->Draw();
    
    right->cd();

    TVirtualPad* right_up = new TPad("right_up","right_up",0.005,0.505,0.995,0.995,0,0,0);
    TVirtualPad* right_down = new TPad("right_down","right_down",0.005,0.005,0.995,0.495,0,0,0);


    right_up->Draw();
    right_down->Draw();

    // second pad, nearby and triggering ratios
    right_up->cd();

    TMultiGraph *gFill = new TMultiGraph("fillings","Fillings");
    TGraph *gRatio = new TGraph(nsteps,theta, ratio);
    gRatio->SetTitle("Ratio of photons triggering/total ?");
    gRatio->SetLineColor(215); 
    gRatio->SetLineWidth(3); 

    TGraph *gFrac = new TGraph(nsteps,theta, fraction);
    gFrac->SetLineColor(210); 
    gFrac->SetLineWidth(3); 
    
    gFill->Add(gRatio,"LP");
    gFill->Add(gFrac,"LP");
    gFill->Draw("A");

    legend = new TLegend(0.7,0.81,0.99,0.99);
    legend->AddEntry(gRatio, "N_{nearby}/N_{FS}","l");
    legend->AddEntry(gFrac,  "N_{trig}/N_{FS}","l");
    legend->Draw();

    // third pad, rms
    right_down->cd();
    TMultiGraph* gMultiRMS= new TMultiGraph("rmss","Point Spread Function RMS");
    TGraph *gRMS = new TGraph(nsteps,theta, rms);
    TGraph *gRMSX = new TGraph(nsteps,theta, rmsX);
    TGraph *gRMSY = new TGraph(nsteps,theta, rmsY);
    TGraph *gRMSfrac = new TGraph(nsteps,theta, rmsfrac);

    gRMS->SetLineColor(222);
    gRMS->SetLineWidth(3); 
    gRMSX->SetLineColor(226);
    gRMSX->SetLineWidth(3); 
    gRMSY->SetLineColor(219);
    gRMSY->SetLineWidth(3); 
    gRMSfrac->SetLineColor(kRed);
    gRMSfrac->SetLineWidth(3); 

    gMultiRMS->Add(gRMS,"LP");
    gMultiRMS->Add(gRMSX,"LP");
    gMultiRMS->Add(gRMSY,"LP");
    gMultiRMS->Add(gRMSfrac,"LP");

    gMultiRMS->Draw("A");
    
    legend = new TLegend(0.7,0.7,0.99,0.99);
    legend->AddEntry(gRMS,  "Average RMS","l");
    legend->AddEntry(gRMSX, "RMS along X","l");
    legend->AddEntry(gRMSY, "RMS along Y","l");
    legend->AddEntry(gRMSfrac, "RMS to match trig eff","l");
    legend->Draw();

    delete [] theta;
    delete [] totEff;
    delete [] trigEff;
    delete [] ratio;
    delete [] rms;
    delete [] rmsX;
    delete [] rmsY;
    delete [] fraction;
    delete [] rmsfrac;
}
