// $Id: LowtranManager.cc,v 1.9 2006/03/24 18:48:45 thea Exp $
// Author:    2006/03/15

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: LowtranManager                                                       *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// LowtranManager
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   fMakeTape5Unique: if yes the tape5 name is randomized over a basename
//   "LwT5_", in the current tmp directory.  The name is followed by 6 random
//   chars to make it unique.
//

#include "LowtranManager.hh"
#include "EsafSys.hh"
#include <TSystem.h>
#include <TMath.h>

LowtranManager* LowtranManager::fgMe = NULL;

ClassImp(LowtranManager)

struct LowtranFiles {
    char Tape5[500];
    char Tape6[500];
    char Tape7[500];
    char Tape8[500];
}; 

// Fortran interfaces
extern "C"{

// calculate transmission between a point and EUSO
void esaflwtrans_(Int_t* init, Float_t* altitude, Float_t* angle, Float_t trans[100]);

// calculate transmission between two points, expressed in lowtran frame
void esaflwtrans_v2_(Int_t* init, Float_t* h1, Float_t* h2, Float_t* theta);
}

extern "C"{
struct LowtranTransferOutput tuple_;
struct LowtranFiles useroutput_;
}

//_____________________________________________________________________________
LowtranManager::LowtranManager() {
    //
    // Constructor
    //

    fTape5 = Conf()->GetStrPath("LowtranManager.fTape5").c_str();
    fTape6 = Conf()->GetStrPath("LowtranManager.fTape6").c_str();
    fTape7 = Conf()->GetStrPath("LowtranManager.fTape7").c_str();
    fTape8 = Conf()->GetStrPath("LowtranManager.fTape8").c_str();

    if ( Conf()->GetBool("LowtranManager.fMakeTape5Unique") ) {
        
        TString dir = gSystem->DirName(fTape5);
        char* tmpstr = tempnam(dir.Data(),"LwT5_");
        fTape5 = tmpstr;
        delete tmpstr;
    }

    memset(useroutput_.Tape5,' ',500);
    memset(useroutput_.Tape6,' ',500);
    memset(useroutput_.Tape7,' ',500);
    memset(useroutput_.Tape8,' ',500);

    SyncTapeNames();
}


//_____________________________________________________________________________
LowtranManager::~LowtranManager() {
    //
    // Destructor
    //
}

//______________________________________________________________________________
LowtranManager* LowtranManager::Get() {
    //
    //
    //
    if (!fgMe) fgMe = new LowtranManager;

    return fgMe;
}

//______________________________________________________________________________
void LowtranManager::SyncTapeNames() {
    //
    //
    //
    
    memcpy(useroutput_.Tape5,fTape5.Data(),fTape5.Length());
    memcpy(useroutput_.Tape6,fTape6.Data(),fTape6.Length());
    memcpy(useroutput_.Tape7,fTape7.Data(),fTape7.Length());
    memcpy(useroutput_.Tape8,fTape8.Data(),fTape8.Length());
}

//______________________________________________________________________________
Bool_t LowtranManager::InitLowtran() {
    // 
    // Lowtran initialisation
    //

    FileStat_t stat;
    if ( gSystem->GetPathInfo(fTape5.Data(), stat) != 0) {
        Msg(EsafMsg::Warning) << "Tape5 datacard " << fTape5 << " does not exists." << MsgDispatch;
        return kFALSE;
    }
 
    // fill the common block
    SyncTapeNames();

    Msg(EsafMsg::Debug) << "Tape5: " << fTape5 << MsgDispatch;
    Msg(EsafMsg::Debug) << "Tape6: " << fTape6 << MsgDispatch;
    Msg(EsafMsg::Debug) << "Tape7: " << fTape7 << MsgDispatch;
    Msg(EsafMsg::Debug) << "Tape8: " << fTape8 << MsgDispatch;

    Float_t trans[100];
    Int_t flag(0);
    Float_t h(0),theta(0);
    esaflwtrans_(&flag,&h,&theta,trans);

    return kTRUE;
}

//______________________________________________________________________________
Bool_t LowtranManager::LwTrans(Int_t init, Float_t altitude, Float_t angle, Float_t* trans) {
    //
    //  Subroutine to calcul the radiative transfert throw the atmosphere
    //  Based on the LOWTRAN7 program
    // 
    //  Input parameters :
    //  ------------------
    //  INIT :  flag for initialisation of lowtran
    //          0 initialisation of atmosphere's parameters and conditions
    //              ( defined in the lowtran.input file )
    //          1 initialisation of geometrical conditions only when
    //            atmosphere's parameters have already been defined
    // 
    //  USR_ALT, USR_ANGLE : altitude of the point in km
    //                       angle theta compare to Nadir (0=vertical)
    //                     origine on ground at Nadir (EUSO = 0.,0.,400.)
    // 
    //  Output parameters :
    //  -------------------
    //  TRANS(lambda) : total transmission from a point at alt usr_alt to a
    //  point above the atmosphere in the theta direction for differents
    //  wavelenghts (no more than 100 and defined in the lowtran.input file)
    //  Then, the transmission can be multiplied.
    // 
    //  Author : Anne Stutz
    //  Last revised by Corinne Berat, August 2002



    esaflwtrans_(&init, &altitude, &angle, trans);
    LoadLwtrnOutput(); 
    return kTRUE;

}

//______________________________________________________________________________
Bool_t LowtranManager::LwTrans_V2(Int_t init, Float_t h1, Float_t h2, Float_t theta) {
    //
    //     Subroutine to calcul the radiative transfert throw the atmosphere
    //     Based on the LOWTRAN7 program
    //
    //     Input parameters :
    //     ------------------
    //     INIT :  flag for initialisation of lowtran
    //             0 initialisation of atmosphere's parameters and conditions
    //                 ( defined in the lowtran.input file )
    //             1 initialisation of geometrical conditions only when
    //               atmosphere's parameters have already been defined
    //
    //     USR_H1, USR_H2 : altitude of the 2 points in km
    //          USR_ANGLE : angle theta compare to Nadir (0=vertical)
    //                        origine on ground at Nadir (EUSO = 0.,0.,400.)
    //
    //     Output parameters :
    //     -------------------
    //     compare to LWTRANS, all the ouputs are available via the
    //     common block "tuple"
    //
    //     Author : Anne Stutz
    //     Last revised by Corinne Berat, April 2004

    esaflwtrans_v2_(&init, &h1, &h2, &theta);
    LoadLwtrnOutput();
    return kTRUE;
}

//______________________________________________________________________________
void LowtranManager::LoadLwtrnOutput() {
    //
    //
    //

    fR = tuple_.R;
    fAltitude = tuple_.Altitude;
    fAngle = tuple_.Angle;

    Int_t len = 100;
    fWaveLength.Set(len,tuple_.WaveLength);
    fTransmission.Set(len,tuple_.Transmission);
    fOzone.Set(len,tuple_.Ozone);
    fRayl.Set(len,tuple_.Rayl);
    fAerosol.Set(len,tuple_.Aerosol);

}

//______________________________________________________________________________
void LowtranManager::GetWaveLength(Float_t *array, Int_t n) const {

    Int_t nMin = TMath::Min(n,fWaveLength.GetSize());
    memcpy(array,fWaveLength.GetArray(),sizeof(Float_t)*nMin);
}

//______________________________________________________________________________
void LowtranManager::GetTransmission(Float_t *array, Int_t n) const {

    Int_t nMin = TMath::Min(n,fTransmission.GetSize());
    memcpy(array,fTransmission.GetArray(),sizeof(Float_t)*nMin);
}

//______________________________________________________________________________
void LowtranManager::GetOzone(Float_t *array, Int_t n) const {

    Int_t nMin = TMath::Min(n,fOzone.GetSize());
    memcpy(array,fOzone.GetArray(),sizeof(Float_t)*nMin);
}

//______________________________________________________________________________
void LowtranManager::GetRayl(Float_t *array, Int_t n) const {

    Int_t nMin = TMath::Min(n,fRayl.GetSize());
    memcpy(array,fRayl.GetArray(),sizeof(Float_t)*nMin);
}

//______________________________________________________________________________
void LowtranManager::GetAerosol(Float_t *array, Int_t n) const {

    Int_t nMin = TMath::Min(n,fAerosol.GetSize());
    memcpy(array,fAerosol.GetArray(),sizeof(Float_t)*nMin);
}

//______________________________________________________________________________
void LowtranManager::GetTransferOutput( LowtranTransferOutput& o ) const {

    o.R = fR;
    o.Altitude = fAltitude;
    o.Angle = fAngle;

    Int_t len = 100;
    size_t size = sizeof(Float_t)*len;
    memcpy(o.WaveLength, fWaveLength.fArray, size);
    memcpy(o.Transmission, fTransmission.fArray, size);
    memcpy(o.Ozone, fOzone.fArray, size);
    memcpy(o.Rayl, fRayl.fArray, size);
    memcpy(o.Aerosol, fAerosol.fArray, size);
}
