// $Id: RecoFramework.cc,v 1.50 2006/03/28 09:13:43 thea Exp $
// Author: Marco Pallavicini Oct, 16 2003
// 
/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: RecoFramework                                                        *
 *  Package: framework                                                       *
 *  Coordinator: Marco.Pallavicini                                           *
 *                                                                           *
 *****************************************************************************/

#include "RecoFramework.hh"
#include "Config.hh"
#include "ModuleFactory.hh"
#include "RecoModule.hh"
#include "InputModule.hh"
#include "RecoEvent.hh"
#include "RecoRootEvent.hh"
#include "TruthModule.hh"
#include "EsafMsgDispatcher.hh"
#include "EsafSys.hh"

#include <list>
#include <TTree.h>
#include <TFile.h>
#include <TObjectTable.h>
#include <TDirectory.h>
#include <TBenchmark.h>
#include <TSystem.h>
#include <TTimeStamp.h>

ClassImp(RecoFramework)

//_____________________________________________________________________________
RecoFramework::RecoFramework(int argc, char** argv) : EsafConfigurable(), EsafMsgSource(), fRootFile(NULL) {
    //
    // Costructor
    //

    Bool_t isLocal(kFALSE); //FIXME: this should be handled by a CmdLineParser object.
    string str;
    for (Int_t i(0); i<argc; str=argv[i++])
        if (str == "--local")
            isLocal = kTRUE;

    const char esafsys[]="ESAFSYS";
    string esafCfgPath;

    TString esafPath;
    if ( gSystem->Getenv(esafsys) && !isLocal )
        esafPath = gSystem->Getenv(esafsys);
    else
        esafPath = gSystem->WorkingDirectory();

    gEsafSys = new EsafSys(esafPath);


    esafCfgPath = gEsafSys->GetCfgPath();

    ConfigFileParser::ChangeConfigDir(esafCfgPath);

    // initialize message dispatcher
    EsafMsgDispatcher::Get()->LoadConfig();
    
    // load standard configs
    Config::Get()->LoadStandardConfigs();


    ParseCommandLine(argc,argv);
    
    Init();

    // get file name with module list
    string sName = Conf()->GetStrPath("RecoFramework.ModuleFile");
    
    // build factory 
    ModuleFactory factory( sName );
    
    // get unique input module
    fInputModule = factory.GetInputModule();
    if ( fInputModule == NULL ) {
        Msg(EsafMsg::Panic) << "Cannot run without input module"<< MsgDispatch;
    }
    
    // get modules
    Int_t counter=0;
    while ( RecoModule *aModule = factory.GetModule() ) {
        Msg(EsafMsg::Info) <<  "Loading module : " << aModule->GetName() << MsgDispatch;
        fModules.push_back(aModule);
        counter++;
    }
    Msg(EsafMsg::Info) << "RecoFramework: " << counter << " modules loaded" << MsgDispatch;
    fCurrentEvent = 0;

}

//_____________________________________________________________________________
RecoFramework::~RecoFramework() {
    //
    // Destructor
    // 

    End();
}

//_____________________________________________________________________________
void RecoFramework::ParseCommandLine(int& argc, char** argv) {
    //
    // Parse a command line
    //

//    list<string> usrCfgs;
//    list< pair<string,string> > cmdArgs;
    
    for(Int_t i=1; i<argc; i++)
        if (strncmp(argv[i],"--",2)==0) {
            // Ok, it seems to be user-defined var
            string s = argv[i]+2;

            if ( s=="local" ) {
                Msg(EsafMsg::Info) << "ESAF-Reco running from the Current Working Directory" << MsgDispatch;
            } else if ( s.substr(0,7) == "usrcfg=") {
                fUsrCfgs.push_back(s.substr(7));
            } else if ( s.substr(0,7) == "stdcfg=" ) {
                //Standard Configuration
                string stdcfg = s.substr(7);
                if ( !Config::Get()->UseConfig( stdcfg ) )
                    FatalError(stdcfg+" standard config not defined");

            } else if ( s.substr(0,8) == "outpath=" ) {
                //Standard Configuration
                 fOutPath = s.substr(8);
            } else if ( s.substr(0,8) == "outname=" ) {
                //Standard Configuration
                 fOutBaseName = s.substr(8);

            }  else if ( s.substr(0,6) == "first=") {
                string key = "RootInputModule";
                string value = "RootInputModule.FirstEvent=";
                value += s.substr(6);
                fUsrKeys.push_back(key);	
                fUsrValues.push_back(value);
                //                Config::Get()->AddUserDefinition(key,value);
            } else if ( s.substr(0,5) == "last=") {
                string key = "RootInputModule";
                string value = "RootInputModule.LastEvent=";
                value += s.substr(5);
                fUsrKeys.push_back(key);	
                fUsrValues.push_back(value);
                //                Config::Get()->AddUserDefinition(key,value);

            } else{
                size_t pos = s.find('.');
                if ( pos == string::npos )
                    FatalError("Invalid parameter "+string(argv[i]));
                else {
                    // user defined variable syntax
                    // ClassName::VarName
                    fUsrKeys.push_back(s.substr(0,pos));	
                    fUsrValues.push_back(s);
                }
            }

        } else if ( i == argc-1) {
            // 
            // filename must be the last parameter
            // 


            Msg(EsafMsg::Info) << "Filename set to " << argv[i]  << MsgDispatch;
            string key = "RootInputModule";
            string value = "RootInputModule.FileName=";
            value+=argv[i];
            fUsrKeys.push_back(key);	
            fUsrValues.push_back(value);
            //            Config::Get()->AddUserDefinition(key,value);

        } else {
             Msg(EsafMsg::Warning) << "Offending argument " << argv[i] << MsgDispatch;
             Msg(EsafMsg::Info) << argv[0] << " usage:" << MsgDispatch;
             Msg(EsafMsg::Info) << MsgDispatch;
             Msg(EsafMsg::Info) << "\t"<< argv[0] << " [ --usrcfg=<file.cfg> --ClassName.VarName=<Value> ] [ file ]" << MsgDispatch;
             Msg(EsafMsg::Info) << MsgDispatch;
             
             FatalError("Reco Terminated");
        }



    // load user definition file first
    vector<string>::const_iterator itCfg;
    for( itCfg=fUsrCfgs.begin(); itCfg!=fUsrCfgs.end(); itCfg++)
        Config::Get()->LoadUserConfig(itCfg->c_str());

    // the apply custom parameters
    for (size_t i(0); i< fUsrKeys.size(); i++) {
        Config::Get()->AddUserDefinition(fUsrKeys[i],fUsrValues[i]);
    }

}


//______________________________________________________________________________
void RecoFramework::Init() {
    //
    // Initialize reco
    //

    string filename, rootname, base, dir;

    // FIXME
//    filename = fInputModule->fInputFileName;
    filename = Config::Get()->GetCF("Reco","RootInputModule")->GetStr("RootInputModule.FileName");
    
    string ext= ".root";
    if ( filename.rfind(ext) == ( filename.size()-ext.size() ) )
        filename.resize(filename.size()-ext.size());


    if ( !(fOutBaseName.size()) )
        fOutBaseName = gSystem->BaseName(filename.c_str());

    //
    // Strip the extension
    // 

    if ( !(fOutPath.size()))
        fOutPath = gSystem->DirName(filename.c_str());
    
    gEsafSys->SetOutPath(fOutPath.c_str());

    string abs_outpath = gEsafSys->AbsolutePath( gEsafSys->GetOutPath());
    
    fFileNames[kRoot] = abs_outpath+fOutBaseName+".reco.root";
    fFileNames[kLog]  = abs_outpath+fOutBaseName+".log";
    fFileNames[kDump] = abs_outpath+fOutBaseName+".CfgDump.cfg";
    
    EsafMsgDispatcher::Get()->OpenLog(fFileNames[kLog]);

    Msg(EsafMsg::Info) << "Logging to file "+fFileNames[kLog]+
        " started" << MsgDispatch;

    // header data
    UInt_t len = 80;

    TString head1('_',80); head1[0]=' '; head1[len-1]=' ';
    TString head2(' ',80); head2[0]='|'; head2[len-1]='|';
    TString head3('_',80); head3[0]='|'; head3[len-1]='|';
    TString line, msg;

    Msg(EsafMsg::Info) << head1 << MsgDispatch;
    Msg(EsafMsg::Info) << head2 << MsgDispatch;

    line = head2;
    msg  = "ESAF Run Starting";
    line.Replace(3,msg.Length(),msg.Data(),msg.Length());

    Msg(EsafMsg::Info) << line << MsgDispatch;

    TTimeStamp tStart;
    line = head2;
    msg  = Form("Local time: %s",tStart.AsString("l"));
    line.Replace(5,msg.Length(),msg.Data(),msg.Length());

    Msg(EsafMsg::Info) << line << MsgDispatch;

    Msg(EsafMsg::Info) << head3 << MsgDispatch;
    Msg(EsafMsg::Info) << MsgDispatch;
    

    MsgForm(EsafMsg::Info,"ESAF Environment:");
    Msg(EsafMsg::Info) << MsgDispatch;
    MsgForm(EsafMsg::Info,"   Current Directory: %s",gSystem->WorkingDirectory());
    MsgForm(EsafMsg::Info,"   Reco Config Path : %s",gEsafSys->GetCfgPath());
    MsgForm(EsafMsg::Info,"   Reco Auxilar Path: %s",gEsafSys->GetAuxPath());
    MsgForm(EsafMsg::Info,"   Reco Tmp Path    : %s",gEsafSys->GetTmpPath());
    MsgForm(EsafMsg::Info,"   Reco Output Path : %s",gEsafSys->GetOutPath());
    MsgForm(EsafMsg::Info,"   Host Machine     : %s",gSystem->HostName());
    Msg(EsafMsg::Info) << MsgDispatch;

 
    // set config dump
    Config::Get()->SetCfgDump( fFileNames[kDump].c_str() );

    vector<string>::const_iterator itCfg;
    for( itCfg=fUsrCfgs.begin(); itCfg!=fUsrCfgs.end(); itCfg++)
        Msg(EsafMsg::Debug) << "Using custom config: "+(*itCfg)<< MsgDispatch;

    // add the user defined variables to the logfile
    for (size_t i(0); i< fUsrKeys.size(); i++) 
        Msg(EsafMsg::Debug) << "Cmd line option: class "+
            fUsrKeys[i]+", value "+fUsrValues[i] << MsgDispatch;
}

//______________________________________________________________________________
void RecoFramework::End() {
    // save trees and close file
    CloseRoot();

    // Dump config file
    Config::Get()->SaveConfig(fFileNames[kDump].c_str());

    UInt_t len = 80;

    TString head1('_',80); head1[0]=' '; head1[len-1]=' ';
    TString head2(' ',80); head2[0]='|'; head2[len-1]='|';
    TString head3('_',80); head3[0]='|'; head3[len-1]='|';
    TString line, msg;

    Msg(EsafMsg::Info) << head1 << MsgDispatch;
    Msg(EsafMsg::Info) << head2 << MsgDispatch;

    line = head2;
    msg  = "ESAF Run Completed";
    line.Replace(3,msg.Length(),msg.Data(),msg.Length());

    Msg(EsafMsg::Info) << line << MsgDispatch;

    TTimeStamp tEnd;
    line = head2;
    msg  = Form("Local time: %s",tEnd.AsString("l"));
    line.Replace(5,msg.Length(),msg.Data(),msg.Length());

    Msg(EsafMsg::Info) << line << MsgDispatch;

    Msg(EsafMsg::Info) << head3 << MsgDispatch;
    Msg(EsafMsg::Info) << MsgDispatch;
}
    
//_____________________________________________________________________________
void RecoFramework::Execute() {
    //
    // Execute a complete run
    //

    string dumpRootEvent = Conf()->GetStr("RecoFramework.DumpRootEvent");

    // init input module
    fInputModule->Init();

    // init all modules
    for( Int_t i=0; i<(Int_t)fModules.size(); i++ ) {
        if ( !fModules[i]->Init() ) {
            Msg(EsafMsg::Panic) << "Module " << fModules[i]->GetName() << " failed" << MsgDispatch;
        }
    }

    OpenRoot();

    TString jobname = Form("Reconstruction Job");
    TBenchmark bJob;
    bJob.Start(jobname);

    // run
    while ( RecoEvent *anEvent = fInputModule->GetEvent() ) {

        //-----------------------------------------------------
        UInt_t len = 80;
        TString divider('_',len);
        TString head1('_',len); head1[0]=' '; head1[len-1]=' ';
        TString head2(' ',len); head2[0]='|'; head2[len-1]='|';
        TString head3('_',len); head3[0]='|'; head3[len-1]='|';
        TString line, msg;

        msg = Form("Event %d Started",fCurrentEvent);
        line = head2;
        line.Replace(3,msg.Length(),msg.Data(),msg.Length());

        Msg(EsafMsg::Info) << head1 << MsgDispatch;
        Msg(EsafMsg::Info) << head2 << MsgDispatch;
        Msg(EsafMsg::Info) << line << MsgDispatch;
        Msg(EsafMsg::Info) << head2 << MsgDispatch;
        Msg(EsafMsg::Info) << MsgDispatch;
        //-----------------------------------------------------


        fRecoRootEvent->Clear();
        if (fInputModule->GetName() == "Root") 
            fInputModule->SaveRootData(fRecoRootEvent);
        if ( anEvent->GetHeader().GetNumActiveFee() != 0 ) {
            TBenchmark bEvent;
            TString s = Form("Event %d",fCurrentEvent);
            bEvent.Start(s);
            // execute all modules
            for( Int_t i=0; i<(Int_t)fModules.size(); i++) {
                if ( ! fModules[i]->PreProcess() )
                    break;
                if ( ! fModules[i]->Process( anEvent ) )
                    break;
                if ( ! fModules[i]->PostProcess() )
                    break;
                if ( ! fModules[i]->SaveRootData(fRecoRootEvent) )
                    break;
                // write the module data in the event
                anEvent->PutRecoModuleData(fModules[i]->GetName(), fModules[i]->GetData());
            }
            if (dumpRootEvent == "yes" ) fRecoRootEvent->Dump();
            fRecoTree->Fill();
            // clean up
            fInputModule->DestroyEvent();            // delete input event
            for( Int_t i=0; i<(Int_t)fModules.size(); i++) {
                fModules[i]->UserMemoryClean();      // delete user objects
                fModules[i]->StandardMemoryClean();  // delete Int_t and Double_t maps
            }
            bEvent.Stop(s);

            //-----------------------------------------------------
            msg = Form("Event %d Completed, REAL=%6.2f s  CPU=%6.2f s",
                    fCurrentEvent,
                    bEvent.GetRealTime(s), bEvent.GetCpuTime(s) );
            line.Replace(3,msg.Length(),msg.Data(),msg.Length());

            Msg(EsafMsg::Info) << MsgDispatch;
            Msg(EsafMsg::Info) << head2 << MsgDispatch;
            Msg(EsafMsg::Info) << line << MsgDispatch;
            Msg(EsafMsg::Info) << head3 << MsgDispatch;
            Msg(EsafMsg::Info) << MsgDispatch;
            //----------------------------------------------------
        } else {
            fInputModule->DestroyEvent();

            //-----------------------------------------------------
            msg = Form("Event %d Empty. Skipped", fCurrentEvent);
            line.Replace(3,msg.Length(),msg.Data(),msg.Length());

            Msg(EsafMsg::Info) << MsgDispatch;
            Msg(EsafMsg::Info) << head2 << MsgDispatch;
            Msg(EsafMsg::Info) << line << MsgDispatch;
            Msg(EsafMsg::Info) << head3 << MsgDispatch;
            Msg(EsafMsg::Info) << MsgDispatch;
            //----------------------------------------------------}

    }

    fCurrentEvent++;


} // end event loop

    // end all modules
    fInputModule->Done();
    for( Int_t i=0; i<(Int_t)fModules.size(); i++) {
        fModules[i]->Done();
    }

    bJob.Stop(jobname);
    MsgForm(EsafMsg::Info,"Average time per event:  REAL=%6.2f s   CPU=%6.2f s",
            bJob.GetRealTime(jobname)/fCurrentEvent, (bJob.GetCpuTime(jobname)/fCurrentEvent) );  
    MsgForm(EsafMsg::Info,"Total time:              REAL=%6.2f s   CPU=%6.2f s",
            bJob.GetRealTime(jobname), bJob.GetCpuTime(jobname));

}
//_____________________________________________________________________________
void RecoFramework::Dump( ostream& os ) const {
    //
    // Dump the module list
    //
    
    Msg(EsafMsg::Debug) << "Framework Dump" << MsgDispatch;
    Msg(EsafMsg::Debug) << "The following Modules where loaded:" << MsgDispatch;
    Msg(EsafMsg::Debug) << "InputModule: " << fInputModule->GetName() << MsgDispatch;
    
    for( Int_t i=0; i<(Int_t)fModules.size(); i++ ) {
        Msg(EsafMsg::Debug) << "Module: " << i+1 << "\t" << fModules[i]->GetName()<< MsgDispatch;
    }
}

//_____________________________________________________________________________
void RecoFramework::OpenRoot() {
    //
    // Open output rootfile
    //

    TString rootname = fFileNames[kRoot];
    TString ext=".reco.root";

    if ( !gSystem->AccessPathName(rootname)  ) {
        // fname.root exists. try to add current date

        MsgForm(EsafMsg::Warning,"Error opening file");
        MsgForm(EsafMsg::Warning,"Probably it already exists or there is no ");
        MsgForm(EsafMsg::Warning,"write access permission to this directory");

        TTimeStamp now;

        UInt_t hour, min, sec, year, month, day;
        now.GetDate(kFALSE,0,&year,&month,&day);
        now.GetTime(kFALSE,0,&hour,&min,&sec);
        TString time = Form(".%4d-%02d-%02d-%02dh%02dm%02ds",year,month,day,hour,min, sec);
        rootname.Insert(rootname.Length()-ext.Length(),time);

    }
    

    fRootFile = new TFile(rootname, "CREATE");
    if (fRootFile->IsZombie()) 
        MsgForm(EsafMsg::Panic,"File %s cannot be opened",rootname.Data());



    Msg(EsafMsg::Info) <<  "Root file " << rootname << " successfully opened" << MsgDispatch;
    fRecoTree = new TTree("recotree","RecoTree");
    CreateRecoRootEvent();
    fRecoTree->Branch("events","RecoRootEvent",&fRecoRootEvent);
}

//_____________________________________________________________________________
void RecoFramework::CreateRecoRootEvent() {
    //
    // Create RecoRootEvent
    //
    fRecoRootEvent = new RecoRootEvent();
}

//_____________________________________________________________________________
void RecoFramework::CloseRoot() {
    // 
    // Close output rootfile
    // 
    if ( fRootFile ) {
        fRecoTree->Write();
        fRecoTree->AutoSave("SelfSave");
        Msg(EsafMsg::Info) << "************************ " << setw(4) << 
            fRecoTree->GetEntries() << " events read ************************" << 
            MsgDispatch;
        Msg(EsafMsg::Info) << MsgDispatch;
        Msg(EsafMsg::Info) << fRecoTree->GetEntries() << " entries are written into " << fRootFile->GetName()<< MsgDispatch;
        fRootFile->Close();
        delete fRootFile;
    }
}
