// ESAF : Euso Simulation and Analysis Framework
// $Id: ProfileModule.hh,v 1.5 2005/11/23 16:02:40 moreggia Exp $
// Author: Anne Stutz   Jun,  6 2005

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ProfileModule                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

#ifndef __PROFILEMODULE_HH_
#define __PROFILEMODULE_HH_

#include "euso.hh"
#include "RecoModule.hh"
#include "EarthVector.hh"
#include "TArrayD.h"
#include "TAxis.h"
#include <TH1F.h>

class RecoEvent;
class RecoRootEvent;

enum ProfileInput {FromPupilTruth, FromFluoTrackTruth, FromShowerTrackTruth};

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// ProfileModule                                                              //
//                                                                            //  
// <brief class description>                                                  //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////

class ProfileModule : public RecoModule  {
public:
    // ctor
    ProfileModule();
    // dtor
    virtual ~ProfileModule();

    // Init method: called at the beginning of a run
    Bool_t Init();

    // called before each event process
    Bool_t PreProcess();

    // event processing
    Bool_t Process( RecoEvent* );

    // called after processing an event
    Bool_t PostProcess();

    // save root output
    virtual Bool_t SaveRootData( RecoRootEvent* );

    // called at the end of each run
    Bool_t Done();

    // memory clean
    void UserMemoryClean();

    // load previous modules
    void LoadModules();

    // photons on pupil -> fluorescence track
    void RecoFluoTrack();

    // fluorescence track -> shower profile
    void RecoShowerProfile();

    // Get track coordinate corresponding to a given time on pupil 
    Double_t GetTrackCoordinate(Double_t t_on_p); 

    EsafConfigClass(Reco,ProfileModule)


private:
   
    RecoEvent *fEvent;                    // event being analysed
    ProfileInput fInputType;              // inputs used


////// internal set and get methods
    inline void SetArraysSize( Int_t size );
    inline void SetFluoMaxPos(const EarthVector&);
    inline Double_t GetFluoHmax() const {return fFluoMaxPos.Zv();}
    inline const EarthVector& EUSO() const {return fEUSO;}
    inline Double_t EusoRadius() const {return fEusoRadius;}
    inline Double_t EusoOmega(const EarthVector& v);
   
   
////// TOOL FUNCTIONS
    // fit fluo profile on pupil, parameters stored in fPar
    void FitFluoOnPupil();
    
    // fit shower profile, with required parametrization
    // GHF, GIL, GFA parametrizations available
    void FitShowerProfile(string type);
    // - parametrization used to fit shower profile
    // - for shower profile fit
    string fParamFormula;
    string fPrimaryType;
    
    // Get fluo mean transmission
    Double_t FluoMeanTransmission(const EarthVector& pos);
    


////// Inputs (from previous module or from simulation)
    Double_t fTheta;                      // local zenith angle (not in MES)
    Double_t fPhi;                        // azimuth zenith angle (not in MES)
    EarthVector fEarthImpact;             // normally used
    EarthVector fFluoMaxPos;              // used if no impact
    Bool_t f3Dref;                        // true if earthimpact used
    Double_t fGTUlength;
    Int_t fNGtu;

////// parameters for 3D reco
    EarthVector fEUSO;                   // detector position
    EarthVector fTOA;                    // primary entering position in atmosphere
    Double_t fEusoRadius;                 // detector radius
    EarthVector fTrackDir;               // shower track 3D-direction (in MES)
    EarthVector fImpactParameter;        // position of shower impact parameter (on the track)
    Double_t fLhmax;                      // dist(EUSO,Fluomax)
    Double_t fLh;                         // shower impact parameter
    Double_t fSh;                         // shower impact parameter position, expressed in tracklength coordinate
    Double_t fShmax;                      // FluoMax position, expressed in tracklength coordinate
    
////// fluo on pupil
    TH1F* fFluoPhP;                      // Xaxis in microseconds
    Double_t fTmin;                       // low edge of first GTU
    Double_t fTmax;                       // up edge of last GTU
    Double_t fTimeAtMax;                  // time of fluo profile maximum
    
////// Fluo track
    TH1F* fFluoTrack;                   // Xaxis in km (see line below)
    TArrayD fFluoTrack_array;           // in km

////// Shower profile
    TH1F* fShowerTrack;                   // Xaxis in km, from TOA
    TArrayD fShowerTrack_array;           // in km, from TOA
    TH1F* fShowerProf;                    // Xaxis in g/cm2
    TArrayD fShowerProf_array;            // in g/cm2
    TH1F* fTrueShowerProf;                // to check algo  //DELETE
    TArrayD fTrueShowerProf_array;        // in g/cm2

////// results
    Double_t fNemax;                      // reco shower maximum amplitude
    Double_t fXmax;                       // reco depth of maximum
    Double_t fX1;                         // reco X1 if GHF used
    Double_t fSigma;                      // reco sigma if GFA used
    Double_t fLambda;                     // reco lambda if GHF used
    Double_t fChiSquare;                  // Chi2 value
    
    Double_t fNemax_err;                  // related error
    Double_t fXmax_err;                   // related error
    Double_t fX1_err;                     // related error

////// profile truth
    Double_t fTrueNemax;                  // true shower maximum amplitude
    Double_t fTrueXmax;                   // true depth of maximum
    Double_t fTrueX1;                     // true depth of maximum


    ClassDef(ProfileModule,0)
};





// inline functions
//_____________________________________________________________________________
inline void ProfileModule::SetArraysSize( Int_t size ) {
    //
    //
    //
    
    fFluoTrack_array.Set(size);
    fShowerProf_array.Set(size);
    fTrueShowerProf_array.Set(size);
    fShowerTrack_array.Set(size);
}

//_____________________________________________________________________________
// inline functions
inline void ProfileModule::SetFluoMaxPos(const EarthVector& pos) {
    //
    // 
    //
    
    fFluoMaxPos.SetXYZ(pos.X(),pos.Y(),pos.Z());
}

//_____________________________________________________________________________
inline Double_t ProfileModule::EusoOmega(const EarthVector& v) {
    //
    //
    //
    
    EarthVector inter = EUSO() - v;
    Double_t th       = fabs(inter.Theta());

    return TMath::TwoPi() * (1 - inter.Mag() / TMath::Sqrt(inter.Mag2() + fEusoRadius*fEusoRadius)) * TMath::Cos(th); 
}


#endif  /* __PROFILEMODULE_HH_ */





