// $Id: ProfileModule.cc,v 1.10 2006/02/19 16:05:34 naumov Exp $
// Author: Anne Stutz   Jun,  6 2005

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: ProfileModule                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// ProfileModule
//
// <extensive class description>
//
//   Config file parameters
//   ======================
//
//   <parameter name>: <parameter description>
//   -Valid options: <available options>
//

#include "ProfileModule.hh"
#include "RecoEvent.hh"
#include "RecoPhotonOnPupilData.hh"
#include "RecoShowerTrackData.hh"
#include "RecoShowerStepData.hh"
#include <TMath.h>
#include <TF1.h>
#include "EConst.hh"
#include "KakimotoFluoCalculator.hh"
#include "EsafSpectrum.hh"
#include "O1_ClearSkyPropagator.hh"  //DELETE
#include "TestGround.hh"
#include "Atmosphere.hh"
#include "RecoRootEvent.hh"
#include "EarthVector.hh"

ClassImp(ProfileModule)

using namespace sou;
using namespace EConst;

// parametrization definitions
//_________________________________________________________________________________________ 
Double_t Profile_GIL(Double_t* x, Double_t* par) {
    //
    // number of electrons at given depth according to GIL analytical formula
    // Catalano et al. ICRC2001
    // GIL stands for Greizen-Ilina-Linsley
    //
    // NB : X unit is g/cm2
    //
    // par[0] : Nmax
    // par[1] : Xmax
    //
    
    Double_t Nmax = par[0];
    Double_t Xmax = par[1];
    Double_t X1   = par[2];

    Double_t x0 = 37.15;
    Double_t t  = (x[0] - X1)/x0;
    Double_t t_max = (Xmax - X1)/x0;
    Double_t Age  = 2*t / (t+t_max);
    Double_t Ne = -1;    
    
    if(Age <= 0) return 0;
    
    Ne = Nmax * exp(t - t_max - 2*t*log(Age));
    
    return Ne;
}

//_________________________________________________________________________________________ 
Double_t Profile_GFA(Double_t* x, Double_t* par) {
    //
    // number of electrons at given depth according to Gaussian Function in Age parameterization.
    // Formulae from C. Song, Astropart. Physics 22(2004)151
    //
    // par[0] : Nmax
    // par[1] : Xmax
    // par[2] : sigma
    //
    
    Double_t Nmax  = par[0];
    Double_t Xmax  = par[1];
    Double_t sigma = par[2];
    Double_t Age = 3*x[0] / (x[0] + 2*Xmax);

    Double_t Ne = -1;
    
    
    if(Age <= 0) return 0;
    
    Ne = Nmax * exp((-1./(2.*sigma*sigma)) * pow(Age - 1.,2));
    
    return Ne;
}

//_________________________________________________________________________________________ 
Double_t Profile_GHF(Double_t* x, Double_t* par) {
    //
    // number of electrons at given depth according to Gaisser Hillas parametrisation
    // Formulae from C. Song, Astropart. Physics 22(2004)151
    //
    // par[0] : Nmax
    // par[1] : Xmax
    // par[2] : lambda
    // par[3] : X0
    //
    
    Double_t Nmax   = par[0];
    Double_t Xmax   = par[1];
    Double_t lambda = par[2];
    Double_t X0     = par[3];

    Double_t X = x[0];
    Double_t Ne = -1;
        
    Ne = Nmax * pow((X-X0)/(Xmax-X0),(Xmax-X0)/lambda) * exp((Xmax-X)/lambda);
    
    return Ne;
}





//_____________________________________________________________________________
ProfileModule::ProfileModule() : RecoModule("Profile") {
    //
    // ctor
    //

}

//_____________________________________________________________________________
ProfileModule::~ProfileModule() {
    //
    // dtor
    //
}

//_____________________________________________________________________________
Bool_t ProfileModule::Init() {
    //
    // initializations
    //

    Msg(EsafMsg::Info) << "Initializing " << MsgDispatch;
    string InputType = Conf()->GetStr( "ProfileModule.fInputType" );
    fParamFormula = Conf()->GetStr("ProfileModule.fParamFormula");
    
    if(InputType == "PupilTruth") {
        Msg(EsafMsg::Info) << "Input for ProfileModule is PhotonsOnPupil truth" << MsgDispatch;
	fInputType = FromPupilTruth;
    }
    else if(InputType == "FluoTrackTruth") {
        Msg(EsafMsg::Info) << "Input for ProfileModule is FluoTrack truth" << MsgDispatch;
	fInputType = FromFluoTrackTruth;
    }
    else if(InputType == "ShowerTruth") {
	Msg(EsafMsg::Info) << "Input for ProfileModule is Shower truth" << MsgDispatch;
	fInputType = FromShowerTrackTruth;
    }
    else
        Msg(EsafMsg::Panic) << "Wrong config value ProfileModule.fInputType " << fInputType << " Setted to truth" << MsgDispatch;
    
    return kTRUE;
}

//_____________________________________________________________________________
Bool_t ProfileModule::PreProcess() {
    //
    //
    //
    
    fEvent          = NULL;
    fTheta          = 0.;
    fPhi            = 0.;
    fNGtu           = 0;
    fTmin           = 0.;
    fTmax           = 0.;
    fTimeAtMax      = 0.;
    fLhmax          = 0.;
    fSh             = 0.;
    fLh             = 0.;
    fNemax          = 0.;
    fXmax           = 0.;
    fSigma          = 0.;
    fX1             = 0.;
    fLambda         = 0.;
    fChiSquare      = 0.;
    fTrueNemax      = 0.;
    fTrueXmax       = 0.;
    fFluoPhP        = NULL;
    fFluoTrack      = NULL;
    fTrueShowerProf = NULL;
    fShowerTrack    = NULL;
    fEusoRadius     = 1250*mm;
    fEUSO.SetXYZ(0,0,430*km);
    Msg(EsafMsg::Warning) <<"SHOULD check if true EUSO position at (0,0,430km) and Radius = 1250mm"<<MsgDispatch;
    
    return kTRUE;
}

//_____________________________________________________________________________
Bool_t ProfileModule::Process(RecoEvent *ev) {
    //
    //
    //
#ifdef DEBUG
    Msg(EsafMsg::Debug)<<"Process"<< MsgDispatch;
#endif

    // manage the profile reconstruction chain
    fEvent = ev; 
    
    // load information from previous modules
    // amd buil
    LoadModules();

    // fit fluo profile on pupil (gaussian fit) -> get timeAtmax
    if(fInputType == FromPupilTruth) {
        FitFluoOnPupil();

	// get track direction in MES (fTheta is not in MES)
	// //TOFIX : fTheta should be defined at TOA, not at first interaction point
	/*
	EarthVector v1 = fEvent->GetHeader().GetTrueInitPos().Unit();
	EarthVector vrot;
	fTrackDir.SetXYZ(1,0,0);
	EarthVector Uz(0,0,1);
	vrot = Uz.Cross(v1);
	fTrackDir.Rotate(v1.Theta(),vrot);
	fTrackDir.Rotate(fPhi,v1);
	vrot = v1.Cross(fTrackDir);
	fTrackDir.Rotate(TMath::Pi()/2+fTheta,vrot);
	fTrackDir.SetTheta(fTrackDir.Theta()+TMath::Pi());
	*/
	//TOFIX
	EarthVector initpos = fEvent->GetHeader().GetTrueInitPos();
	fTrackDir.SetMagThetaPhi(1.,(fFluoMaxPos - initpos).Theta(),(fFluoMaxPos - initpos).Phi());

	// Define some quantities used to translate time on pupil into position along track
	// origin of track coordinates == point corresponding to up-edge of first GTU (on pupil)
	// firstly : calculate ShowerMax coordinate along track
	// then    : the rest follows
	fLhmax            = (EUSO() - fFluoMaxPos).Mag();
	Double_t scalar   = fTrackDir.Dot(EUSO() - fFluoMaxPos);
	Double_t deltaT_c = (fTmin - fTimeAtMax) * Clight();
	fShmax            = (deltaT_c*deltaT_c + 2*deltaT_c*fLhmax) / (2*(fLhmax + deltaT_c - scalar));
	fShmax           *= -1.;
	fSh               = fShmax + scalar;
	fImpactParameter  = fFluoMaxPos + (fSh-fShmax)*fTrackDir;
	fLh               = (EUSO() - fImpactParameter).Mag();
	cout<<"true theta  phi  = "<<fTheta/deg<<"  "<<fPhi/deg<<endl;
	cout<<"theta  phi  mag  = "<<fTrackDir.Theta()/deg<<"  "<<fTrackDir.Phi()/deg<<"  "<<fTrackDir.Mag() <<endl;
	cout<<"Shmax = "<<fShmax/km<<endl;
	cout<<"Sh1 = "<<GetTrackCoordinate(fTmin)/km<<endl;
	cout<<"Shend = "<<GetTrackCoordinate(fTmax)/km<<endl;
	cout<<"showermaxpos.Zv = "<<fFluoMaxPos.Zv()/km<<endl;
	cout<<"showermaxpos = "<<fFluoMaxPos<<endl;
    }
    
    // translate fluo transmitted on pupil into fluo generated along track
    if(fInputType == FromFluoTrackTruth) RecoFluoTrack();
    
    // Iteration in order to estimate fluorescence photons emitted at shower track
    RecoShowerProfile();
    
    // fit shower profile with required parametrization type
    FitShowerProfile(fParamFormula);
        
    return kTRUE;
}

//_____________________________________________________________________________
void ProfileModule::LoadModules()  {
    //
    //
    //
#ifdef DEBUG
          Msg(EsafMsg::Debug)<<"Load Modules"<< MsgDispatch;
#endif
    
    fTheta =  fEvent->GetHeader().GetTrueTheta();
    fPhi =  fEvent->GetHeader().GetTruePhi();
    SetFluoMaxPos(fEvent->GetHeader().GetTrueFluoMaxPos()); // if shifted w.r.t. electrons, it remains on track anyway
    
    // load information from previous modules or from truth
    if(fInputType == FromPupilTruth) { //OLD
        Msg(EsafMsg::Info)<<"Profile reco from PUPIL TRUTH"<< MsgDispatch;
#ifdef DEBUG
        Msg(EsafMsg::Debug)<<" Nb of true photons on pupil "<<fEvent->GetHeader().GetTruePhotonsOnPupil()<< MsgDispatch;
#endif
        RecoPhotonOnPupilData *rph = 0;
        fTmin = HUGE;
        fTmax = 0;
	Double_t fGTUlength = fEvent->GetHeader().GetGtuLength();
        for ( Int_t i=0;i< fEvent->GetHeader().GetTruePhotonsOnPupil();i++) {
            rph = fEvent->GetRecoPhotonOnPupilData(i);
	    // fluo only, in a first time
	    if(rph->GetType()==0) {
	        if ( rph->GetTime()>fTmax ) fTmax = rph->GetTime();
                if ( rph->GetTime()<fTmin ) fTmin = rph->GetTime();            
	    }
        }
        fNGtu = Int_t((fTmax-fTmin)/fGTUlength) + 1;
        Msg(EsafMsg::Debug)<<" Nb of GTU "<<fNGtu <<" from "<< fTmin/microsecond <<" to "<<fTmax/microsecond<<MsgDispatch;
        SetArraysSize(fNGtu+1);
	
	fFluoPhP = (TH1F*)gROOT->FindObject("fluoOnPupil");
	if(!fFluoPhP)
	    fFluoPhP = new TH1F("fluoOnPupil","fluoOnPupil",fNGtu,fTmin/microsecond,(fTmin + fNGtu*fGTUlength)/microsecond);
        for ( Int_t i=0;i< fEvent->GetHeader().GetTruePhotonsOnPupil();i++) {
            rph = fEvent->GetRecoPhotonOnPupilData(i);
	    // fluo only, in a first time
            if(rph->GetType()==0) fFluoPhP->Fill(rph->GetTime()/microsecond);            
        }
    }
    
    
    else if(fInputType == FromFluoTrackTruth) Msg(EsafMsg::Panic) <<fInputType<<" not implemented yet"<< MsgDispatch;
    
    
    else if(fInputType == FromShowerTrackTruth) {
        Msg(EsafMsg::Info)<<"Profile reco from SHOWERtrack TRUTH"<< MsgDispatch;
	
	// get input data
	RecoShowerTrackData* trackdata = fEvent->GetRecoShowerTrackData();
	
	// get profile truth
	fTrueNemax = trackdata->GetNemax();
	fTrueXmax = trackdata->GetXmax();
	fTrueX1 = trackdata->GetX1();

        // extract input data
	fTrackDir = trackdata->GetDir();
        fEarthImpact = trackdata->GetEarthImpact();
	if(fEarthImpact.Zv() > kTolerance) {
	    f3Dref = false;
	    Msg(EsafMsg::Panic) << "Showers without impact not handled yet" << MsgDispatch;
	}
	else f3Dref = true;
	fTOA = Atmosphere::Get()->ImpactAtTOA(fEarthImpact,-fTrackDir,100*km);
#ifdef DEBUG
        Msg(EsafMsg::Debug)<<"TOA = "<<fTOA<< MsgDispatch;
#endif
	const RecoShowerStepData* stepdata = 0;
	Int_t nbsteps = trackdata->GetNumSteps();
	
	// define X-axis
	SetArraysSize(nbsteps+1);
	for(Int_t i=0; i<nbsteps; i++) {
	    stepdata = trackdata->GetStep(i);
	    fShowerTrack_array[i] = (fTOA - stepdata->GetPosi()).Mag();
	    //fTrueShowerProf_array[i] = stepdata->GetXi()*cm2/g; //DEBUG
	}
	fShowerTrack_array[nbsteps] = (fTOA - stepdata->GetPosf()).Mag();
	fShowerTrack = new TH1F("ShowerTrack","ShowerTrack",nbsteps,fShowerTrack_array.GetArray());
	//fTrueShowerProf_array[nbsteps] = stepdata->GetXf()*cm2/g; //DEBUG
	//fTrueShowerProf = new TH1F("TrueShowerProf","TrueShowerProf",nbsteps,fTrueShowerProf_array.GetArray());  //DEBUG
	
	// fill histos
	for(Int_t i=0; i<nbsteps; i++) {
	    stepdata = trackdata->GetStep(i);
	    fShowerTrack->SetBinContent(i+1,stepdata->GetNe());
	    //fTrueShowerProf->SetBinContent(i+1,stepdata->GetNe()); //DEBUG
	}
        //fTrueShowerProf->Write(); //DEBUG
    }
    
    else Msg(EsafMsg::Panic) << "Should not occur, input type :" <<fInputType << MsgDispatch;
}

//_____________________________________________________________________________
void ProfileModule::RecoFluoTrack()  {
    //
    // transform photons on pupil to photons emitted isotropically at shower track
    //
#ifdef DEBUG
          Msg(EsafMsg::Debug)<<"Reco fluo track"<< MsgDispatch;
#endif
        
    // define Xaxis of fluo histo along track
    for ( Int_t i=0; i<fNGtu;i++ ) {
	fFluoTrack_array[i] = GetTrackCoordinate(fFluoPhP->GetBinLowEdge(i+1)*microsecond)/km;
    }
    fFluoTrack_array[fNGtu] = GetTrackCoordinate((fFluoPhP->GetBinLowEdge(fNGtu)+fFluoPhP->GetBinWidth(fNGtu))*microsecond)/km;
    
    // build output histo
    fFluoTrack = (TH1F*)gROOT->FindObject("fluoAtTrack");
    if(!fFluoTrack) fFluoTrack = new TH1F("fluoAtTrack","fluoAtTrack",fNGtu,fFluoTrack_array.GetArray());
        
    // fill output histo
    EarthVector pos;
    Double_t trans = 1.;
    for ( Int_t i=0; i<fNGtu;i++ ) {
        // Get mean position in space corresponding to the GTU bin
	pos = fFluoMaxPos + (fFluoTrack->GetBinCenter(i+1)*km - fShmax)*fTrackDir;
	trans = FluoMeanTransmission(pos);
	fFluoTrack->SetBinContent(i+1,fFluoPhP->GetBinContent(i+1)/EusoOmega(pos)/trans);
    }
    
    //fFluoTrack->Write();//DELETE
    
    // for debugging
    /*
    TArrayD fluoalt_array;
    fluoalt_array.Set(fNGtu+1);
    EarthVector alt;
    for ( Int_t i=0; i<fNGtu;i++ ) {
        alt = fFluoMaxPos + (fFluoTrack_array[i]*km - fShmax)*fTrackDir;
	fluoalt_array[fNGtu - i] = alt.Zv()/km;
	cout<<"altitude["<<fNGtu - i<<"] = "<<fluoalt_array[fNGtu - i]<<endl;
    }
    alt = fFluoMaxPos + (fFluoTrack_array[fNGtu]*km - fShmax)*fTrackDir;
    fluoalt_array[0] = alt.Zv()/km;
    cout<<"altitude[0] = "<<fluoalt_array[0]<<endl;
    TH1F* fFluoAltitude = 0;
    fFluoAltitude = (TH1F*)gROOT->FindObject("FluoAltitude");
    if(!fFluoAltitude) fFluoAltitude = new TH1F("FluoAltitude","FluoAltitude",fNGtu,fluoalt_array.GetArray());
    for ( Int_t i=0; i<fNGtu;i++ ) {
	fFluoAltitude->SetBinContent(fNGtu - i,fFluoTrack->GetBinContent(i+1));
    }
    fFluoAltitude->Write();
    */
}

//_____________________________________________________________________________
void ProfileModule::RecoShowerProfile()  {
    //
    // calculate number of charged particles in each depth bin
    //
#ifdef DEBUG
          Msg(EsafMsg::Debug)<<"Reco shower profile"<< MsgDispatch;
#endif
    
    // get atmosphere description
    const Atmosphere* atmo = Atmosphere::Get();
    
/*	 to be adapted when using LightTruth or pupiltruth //OLD

    // build output histo Xaxis
    EarthVector pos;
    for ( Int_t i=0; i<fNGtu+1;i++ ) {
        pos = fFluoMaxPos + (fFluoTrack_array[i]*km - fShmax)*fTrackDir;
	fShowerProf_array[i] = atmo->Grammage(pos,-fTrackDir,"dir")*cm2/g;
    }
    
    // build output histo
    fShowerProf = (TH1F*)gROOT->FindObject("ShowerProf");
    if(!fShowerProf) fShowerProf = new TH1F("ShowerProf","ShowerProf",fNGtu,fShowerProf_array.GetArray());
    
    // fill output histo
    for ( Int_t i=0; i<fNGtu;i++ )
	fShowerProf->SetBinContent(i+1,fFluoTrack->GetBinContent(i+1)/(4 * fFluoTrack->GetBinWidth(i+1)/m));
    
*/

    // build output histo Xaxis
    EarthVector pos;
    for (Int_t i=0; i<fShowerTrack_array.GetSize(); i++) {
        pos = fTOA + fShowerTrack_array[i]*fTrackDir;
	fShowerProf_array[i] = atmo->Grammage(pos,fTOA)*cm2/g;
    }
    
    // build output histo
    fShowerProf = (TH1F*)gROOT->FindObject("ShowerProf");
    if(!fShowerProf) fShowerProf = new TH1F("ShowerProf","ShowerProf",fShowerTrack->GetNbinsX(),fShowerProf_array.GetArray());
    
    // fill output histo
    for (Int_t i=0; i<fShowerTrack_array.GetSize(); i++) fShowerProf->SetBinContent(i+1,fShowerTrack->GetBinContent(i+1));


    //fShowerProf->Write(); //DEBUG
}

//_____________________________________________________________________________
Double_t ProfileModule::GetTrackCoordinate(Double_t t_on_p) {
    //
    // Get track coordinate corresponding to a given time on pupil
    // Coordinate origin corresponds to the up-edge of first GTU
    //
    
    Double_t rtn = 0;
    Double_t deltaT_c = 0;

    // get position on track as function of time on pupil
    deltaT_c = (t_on_p - fTimeAtMax) * Clight();
    rtn = (fLhmax + fShmax + deltaT_c)*(fLhmax + fShmax + deltaT_c) - fLh*fLh - fSh*fSh;
    rtn /= 2*(deltaT_c + fLhmax + fShmax - fSh);
    
    return rtn;
}

//_____________________________________________________________________________
void ProfileModule::FitFluoOnPupil() {
    //
    // fit fluo profile on pupil with a gaussian
    // get time of maximum, nb of photons at maximum, profile width (sigma)
    //
#ifdef DEBUG
          Msg(EsafMsg::Debug)<<"Fit fluo on pupil"<< MsgDispatch;
#endif
    
    /*
    Double_t* par = new Double_t[3];
    TF1* mygaus = new TF1("mygaus","gaus");   //TOFIX : to delete
    fFluoPhP->Fit("mygaus","Q");
    mygaus->GetParameters(par);
    fFluoPhP->Write(); //DELETE
    fTimeAtMax = par[1]*microsecond;
    
    delete[] par;
    par = 0;    
    */
}

//_____________________________________________________________________________
void ProfileModule::FitShowerProfile(string type) {
    //
    // fit shower profile along track
    //
#ifdef DEBUG
          Msg(EsafMsg::Debug)<<"Fit shower profile"<< MsgDispatch;
#endif
    
    Double_t uplim = fShowerProf->GetBinLowEdge(fShowerProf->GetNbinsX()) + fShowerProf->GetBinWidth(fShowerProf->GetNbinsX());
    Double_t Nmax = fShowerProf->GetMaximum();  //init value
    Double_t Xmax = fShowerProf->GetBinCenter(fShowerProf->GetMaximumBin());  //init value
    Double_t X1   = 40.;  //init value
    TF1* func = 0;

    if(fParamFormula == "GIL")   func = new TF1("ShowerFit",Profile_GIL,0.,uplim,3);
    //if(fParamFormula == "GHF") func = new TF1("ShowerFit",Profile_GHF,0.,uplim,3);
    //if(fParamFormula == "GFA") func = new TF1("ShowerFit",Profile_GFA,0.,uplim,4);
       
#ifdef DEBUG
    Msg(EsafMsg::Debug) <<"(INITIAL)  Nmax  Xmax = "<<Nmax<<"  "<<Xmax<< MsgDispatch;
#endif
    
    func->SetParameters(Nmax,Xmax,X1);
    func->SetParLimits(0,0.5*Nmax,2*Nmax);  //TOFIX
    func->SetParLimits(1,0,uplim);  //TOFIX
    func->SetParLimits(2,0,uplim);  //TOFIX
    
    fShowerProf->Fit("ShowerFit","QNO");
    fNemax     = func->GetParameter(0);
    fNemax_err = func->GetParError(0);
    fXmax      = func->GetParameter(1);
    fXmax_err  = func->GetParError(1);
    fX1        = func->GetParameter(2);
    fX1_err    = func->GetParError(2);
    fChiSquare = func->GetChisquare();
    fLambda = 0.;//TOFIX
    fSigma = 0.;//TOFIX
    //func->Write(); //DELETE
    
#ifdef DEBUG
    Msg(EsafMsg::Debug) <<"Nmax  Xmax  X1  Chisquare = "<<fNemax<<"  "<<fXmax<<"  "<<fX1<<"  "<<fChiSquare<< MsgDispatch;
#endif
    
    SafeDelete(func);
    
}


//_____________________________________________________________________________
Double_t ProfileModule::FluoMeanTransmission(const EarthVector& pos) {
    //
    // calculate mean transmission from a given position on showertrack
    //

    // initializations
    // use of simulation classes -> //TOFIX
    KakimotoFluoCalculator fluocalcul;
    EsafSpectrum fluospec;
    TestGround gnd;
    O1_ClearSkyPropagator* transcalcul = new O1_ClearSkyPropagator(&gnd);
    DetectorGeometry dum;
    dum.SetRadius(EusoRadius());
    dum.SetPos(EUSO());
    transcalcul->CopyDetectorGeometry(&dum,false);
    
    // get fluo spectrum and yield at pos
    // electrons energy taken constant at 80MeV
   // Double_t yield = fluocalcul.GetFluoYield(pos.Zv(),80*MeV,&fluospec);
    Double_t meantrans = 0.;//transcalcul->GoToDetector(fluospec,pos);
    
    delete transcalcul;
    transcalcul = 0;
    
    return meantrans;
    
}

//_____________________________________________________________________________
Bool_t ProfileModule::PostProcess() {
    //
    //
    //

    return kTRUE;
}

//_____________________________________________________________________________
Bool_t ProfileModule::Done() {
    //
    //
    //

    Msg(EsafMsg::Info)<<"ProfileModule done. "<< MsgDispatch;
    return kTRUE;
}

//_____________________________________________________________________________
void ProfileModule::UserMemoryClean()  {
    //
    // delete user objects
    //
    SafeDelete(fFluoPhP);
    SafeDelete(fFluoTrack);
    SafeDelete(fTrueShowerProf);
    SafeDelete(fShowerTrack);
    SafeDelete(fShowerProf);
}

//_____________________________________________________________________________
Bool_t ProfileModule::SaveRootData(RecoRootEvent *fRecoRootEvent) {
    //
    //
    //
#ifdef DEBUG
    Msg(EsafMsg::Debug)<<"Save reco root data"<< MsgDispatch;
#endif
    
    RecoProfile& output = fRecoRootEvent->GetRecoProfile();
    RecoEnergy&  energy = fRecoRootEvent->GetRecoEnergy();
    
    // profile outputs
    output.SetQuality(fChiSquare);
    output.SetXmax(fXmax);
    output.SetNemax(fNemax);
    output.SetSigma(fSigma);
    output.SetX1(fX1);
    output.SetLambda(fLambda);
    output.SetErrorXmax(fXmax_err);
    output.SetErrorNemax(fNemax_err);
    output.SetErrorSigma(0);
    output.SetErrorX1(fX1_err);
    output.SetErrorLambda(0);
    
    energy.SetEnergy(fNemax*1.45*GeV); 
    energy.SetQuality(0);

    return kTRUE;
}



