// $Id: SimpleDetector.cc,v 1.1 2006/03/24 16:23:27 moreggia Exp $
// Author: Sylvain Moreggia   2006/03/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: SimpleDetector                                                           *
 *  Package: <packagename>                                                   *
 *  Coordinator: <coordinator>                                               *
 *                                                                           *
 *****************************************************************************/

//_____________________________________________________________________________
//
// SimpleDetector
//
// <extensive class description>
//
//
// ******* WARNING ********
// Data members of Photon and ParentPhoton have not the std meaning
//
// - ParentPhoton.fType      = type of photon
//                           1: fluo
//                           2: ckov
//
// - Photon.history          = their history IN ATMOSPHERE
//                          -1: none
//                           0: direct
//                           1: reflected
//                           2: rayleigh scat
//                           3: clouds scat
//                           4: aerosols scat
//
// - Photon.fate             = their history IN DETECTOR
//                          -1: outFoV
//                           0: alive
//                           1: absorbed by optics
//                           2: absorbed by BG3
//                           3: absorbed by DQE
//
// - Photon.iterations       = NB of INTERACTIONS in atmosphere (not in detector)
//
//

#include "SimpleDetector.hh"
#include "ESimpleDetector.hh"
#include "DetectorGeometry.hh"
#include "ESimpleDetectorPhotonAdder.hh"
#include "ESimpleDetectorPhotoElecAdder.hh"
#include "EEvent.hh"
#include "Photon.hh"
#include "ParentPhoton.hh"
#include "PhotonsOnPupil.hh"
#include "PhotoElectron.hh"
#include "NumbersFileParser.hh"
#include "EConst.hh"
#include "Pupil.hh"
#include "EsafRandom.hh"
#include "SimpleTelemetry.hh"

ClassImp(SimpleDetector)

using namespace TMath;
using namespace sou;
using namespace EConst;


//_____________________________________________________________________________
SimpleDetector::SimpleDetector() : Detector(), EsafConfigurable(), EsafMsgSource() {
    //
    // Constructor
    //
    // NB : position is set in SimuApplication.cc by reading General/Euso.cfg file
    //
    
    fGeometry = new DetectorGeometry;
    InitDetector();
}

//_____________________________________________________________________________
SimpleDetector::~SimpleDetector() {
    //
    // Destructor
    //
    
    Reset();
    SafeDelete(fGeometry);
}

//_____________________________________________________________________________
void SimpleDetector::Reset() {
    //
    // get ready for next event
    //
        
    map<UInt_t, vector<PhotoElectron*>* >::const_iterator it;
    for(it = fPhotoElectrons.begin(); it != fPhotoElectrons.end(); it++) {    
        vector<PhotoElectron*>* pelist = it->second;
        if(pelist) {
            for(size_t j=0; j < pelist->size(); j++) {
	        PhotoElectron* pe = (*pelist)[j];
	        SafeDelete(pe);
            }
            pelist->clear();
            SafeDelete(pelist);
        }
    }
    fPhotoElectrons.clear();
    
}

//_____________________________________________________________________________
void SimpleDetector::InitDetector() {
    //
    // initialization
    //
    
    // read configs
    fGeometry->SetRadius(Conf()->GetNum("SimpleDetector.fRadius")*sou::m);
    fGeometry->SetFoV(Conf()->GetNum("SimpleDetector.fFoV")*sou::deg);
    fGTUlength = Conf()->GetNum("SimpleDetector.fGTUlength")*sou::nanosecond;
    fPixelSideLength = Conf()->GetNum("SimpleDetector.fPixelSideLength")*sou::m;
    fBgndMode = Conf()->GetStr("SimpleDetector.fBgndMode");
    fBG3type = Conf()->GetStr("SimpleDetector.fBG3type");
    fTriggerLevel = Conf()->GetNum("SimpleDetector.fTriggerLevel");

    // build BG3 filter
    if(fBG3type == "BG3") {
        NumbersFileParser nf("config/Optics/OpticalAdaptor/BG3.dat",3);
        vector<Double_t> wl    = nf.GetCol(0);
        vector<Double_t> trans = nf.GetCol(1);
        vector<Double_t> fake  = nf.GetCol(2);
        for(Int_t i=0; i<400; i++) fBG3filter[i] = trans[i];
    }
    else if(fBG3type == "BG3_coated") {
        NumbersFileParser nf("config/Optics/OpticalAdaptor/BG3_coated.dat",3);
        vector<Double_t> wl    = nf.GetCol(0);
        vector<Double_t> trans = nf.GetCol(1);
        vector<Double_t> fake  = nf.GetCol(2);
        for(Int_t i=0; i<400; i++) fBG3filter[i] = trans[i];
    }
    else Msg(EsafMsg::Panic)<< "<InitDetector> Wrong argument for BG3 type = "<<fBG3type << MsgDispatch;
    
    // build BG3 filter
    NumbersFileParser nf2("config/Electronics/R8900M36Photomultiplier/AverageQuantumEfficiency.dat",2);
    vector<Double_t> wl2    = nf2.GetCol(0);
    vector<Double_t> trans2 = nf2.GetCol(1);
    for(Int_t i=0; i<51; i++) fPMQE[i] = trans2[i];
    
}

//______________________________________________________________________________
Telemetry* SimpleDetector::Get( PhotonsOnPupil* photons ) {
    //
    // Simulate one event
    //
    
    
    // if no photons on pupil
    if ( !photons ) { 
        MsgForm(EsafMsg::Warning,"PhotonsOnPupil is NULL. Photon tracing disabled.");
        return NULL;
    }

    // set usefull quantities
    fNbPixelOnSide = UInt_t(ceil(fGeometry->GetPos().Z() * Tan(fGeometry->GetFoV()) / fPixelSideLength)) + 1;  // +1 to handle round-off in posZ0 calculations
    fFirstPixelX = - fPixelSideLength * fNbPixelOnSide;
    fFirstPixelY = fPixelSideLength * fNbPixelOnSide;
    fNbPixelOnSide *= 2;  // cause X negative AND positive
    Double_t configbgnd = Conf()->GetNum("SimpleDetector.fMeanBgnd");  // in ph/m2/ns/sr
    fMeanBgnd = MeanBgndPerPixelPerGTU(configbgnd);
    Msg(EsafMsg::Info)<< "Mean Background (Nadir) = "<<fMeanBgnd <<" per pixel per GTU" << MsgDispatch;
    
    // reset previous event 
    Reset();

    // wrapper to track the photons onto the real pupil
    Pupil pupil( photons, GetGeometry());
    
    // init
    Photon* ph = 0;
    Int_t fate = 0;
    Int_t Nbph_indet = 0;
    vector<PhotoElectron*> templist;
    EEvent* ev = EEvent::GetCurrent();
    Int_t nTracked=0, nTotal(photons->GetNphotons());
    Msg(EsafMsg::Info) << "Convert photons into PhotoElectrons: " << MsgFlush;
    


   
    
    // loop on photons on pupil
    // 1. Apply optics + electronics filters : photons still alive give a pe-
    while(kTRUE) {
        ph = pupil.GetPhoton();
	if(!ph) break; // end of list
	
	Nbph_indet++;
	
	// update display
        if ( nTracked*50 % nTotal == 0  || (nTracked-1)*50 % nTotal > nTracked*50 % nTotal )
            if ( nTracked*10 % nTotal == 0 || (nTracked-1)*10 % nTotal > nTracked*10 % nTotal ){
                Msg(EsafMsg::Info) << Form("%d",nTracked*100/nTotal) << '%'<< MsgFlush;
            }
            else Msg(EsafMsg::Info) << "." << MsgFlush;
        nTracked++;
	
	
	// 0. transfer atmosphere infos from parentphoton to photon
        ParentPhoton* parentph = ph->parent;
	if(parentph->Type() == 3) parentph->SetType(CerenkovReflected);
	ph->history  = parentph->AtmoHistory();
	ph->iterations = parentph->AtmoNbinter();
	
	
	// 1. FoV handling : if outFoV, tagged accordingly
	if(!fGeometry->IsInFoV(ph->dir)) {
	    ph->fate = -1;
            if( ev ) {
                ESimpleDetectorPhotonAdder a(ph,kTRUE);  
                ev->Fill(a);
            }
	    continue;
	}
	
	
	// 2. optics effects
	fate = ApplyOpticsFilter(*ph);
	if(fate != 0) {
	    ph->fate = fate;
            if( ev ) {
                ESimpleDetectorPhotonAdder a(ph,kTRUE);  
                ev->Fill(a);
            }
	    continue;
	}
	
	
	// 3. Detection quantum efficiency (DQE = QE * CE)
	fate = ApplyDQE(*ph);
	if(fate != 0) {
	    ph->fate = fate;
            if( ev ) {
                ESimpleDetectorPhotonAdder a(ph,kTRUE);  
                ev->Fill(a);
            }
	    continue;
	}
        
        // 4. if photon still alive --> gives a pe-
	if(ph->fate != 0) Msg(EsafMsg::Panic)<< "<Get> Photon fate SHOULD be zero here : "<<ph->fate << MsgDispatch;
	if( ev ) {
            ESimpleDetectorPhotonAdder a(ph,kTRUE);  
            ev->Fill(a);
        }
	templist.push_back(new PhotoElectron(*ph));
    }
    fNbSignalPE = templist.size();
    Msg(EsafMsg::Info) << " -done" << MsgDispatch;
    Msg(EsafMsg::Info)<< "NB of PhotoElectrons given by signal photons = "<<fNbSignalPE << MsgDispatch;
    if(fNbSignalPE == 0) return (Telemetry*)0;
    
    
    // 2. orders pe- chronologically
    //    then find the time of first GTU and ranges pe- in the map using GTU info
    //    Set pixel ID
    SpaceTimePEinfos(templist);
    
    
    // 3. Add background in pixels/GTU where signal photons exist
    //    Apply trigger
    Int_t nbpebgnd = AddBackground();
    Msg(EsafMsg::Info)<< "NB of Background PhotoElectrons added = "<<nbpebgnd << MsgDispatch;
    
    
    // 4. kind of trigger simulation
    //    Tag pe- according to the SNR per pixel per GTU : if (S + B) >= (<B> + fTriggerLevel*sigmaB), pe- is considered triggered
    //    returns nb of triggered pe-
    Int_t nbpetriggered = ApplyTrigger();
    Msg(EsafMsg::Info)<< "NB of Triggered PhotoElectrons = "<<nbpetriggered << MsgDispatch;
    
    
    // 5. fill ESimpleDetector (root output for present detector simu)
    ESimpleDetector* Edet = 0;
    if(ev) Edet = ev->GetSimpleDetector();
    if(Edet) {
        Edet->SetMeanBgnd(fMeanBgnd);
        Edet->SetTriggerLevel(fTriggerLevel);
        Edet->SetNbPixelOnSide(fNbPixelOnSide);
        Edet->SetPixelSideLength(fPixelSideLength);
        Edet->SetFirstPixelXY(fFirstPixelX,fFirstPixelY);
        Edet->SetFirstTime(fFirstPETime);
        Edet->SetLastTime(fLastPETime);
        Edet->SetFirstGTUTime(fFirstGTUTime);
        Edet->SetNbGTUs(fNbGTUs);
    }
    // fill EPhotoElectron list
    map<UInt_t, vector<PhotoElectron*>* >::const_iterator it;
    for(it = fPhotoElectrons.begin(); it != fPhotoElectrons.end(); it++) {    
	vector<PhotoElectron*>* pelist = it->second;

	// if pe- exist in this GTU
	if(pelist) {
	    for(size_t i=0; i < pelist->size(); i++) {
	        ESimpleDetectorPhotoElecAdder a((*pelist)[i],kTRUE);
		ev->Fill(a);
	    }
	}
    }
    
        
    
    return (Telemetry*)0;
}

//______________________________________________________________________________
Int_t SimpleDetector::ApplyOpticsFilter(const Photon& ph) const {
    //
    // Total optical efficiency + BG3
    // return photon's fate value
    //
   
    // inits
    TRandom* rndm = EsafRandom::Get();
    const ParentPhoton* parentph = ph.parent;
    
    // 1. total optical throughput efficiency (taken from OM-DESIGN-001-2)
    //    linear interpolation as a function of theta angle
    Double_t eff = 0.;
    Double_t th = parentph->Theta();
    if(th <= 10*DegToRad())       eff = 0.53 - th * 0.01 / 10*DegToRad();
    else if(th <= 20*DegToRad())  eff = 0.52 - (th - 10*DegToRad()) * 0.02 / 10*DegToRad();
    else if(th <= 30*DegToRad())  eff = 0.50 - (th - 20*DegToRad()) * 0.11 / 10*DegToRad();
    else Msg(EsafMsg::Panic)<< "<ApplyOpticsFilter> Angle on pupil SHOULD NOT be so large = "<<th*RadToDeg() << MsgDispatch;
    
    if(eff < rndm->Rndm()) return 1;


    // 2. BG3 filter
    eff = InterpolateBG3(ph.wl);
    if(eff < rndm->Rndm()) return 2;
    
    // if photon is still alive
    return 0;
}

//______________________________________________________________________________
Int_t SimpleDetector::ApplyDQE(const Photon& ph) const {
    //
    // Detection Quantum Efficiency rules the creation of photoelectrons from incident photons
    // return photon's fate value
    //
    
    // inits
    TRandom* rndm = EsafRandom::Get();
    const ParentPhoton* parentph = ph.parent;
    
    // DQE = CE * QE
    // mean collection efficiency = 0.67025 drawn from config/Electronics/R8900M36Photomultiplier/AverageCollectionEfficiency.dat
    Double_t NORM = 0.67025;
    
    // 1. angular dependence
    Double_t th = parentph->Theta();
    if(th <= 10*DegToRad())       NORM *= 1.     - th                   * 0.03   / 10*DegToRad();
    else if(th <= 20*DegToRad())  NORM *= 0.97   - (th - 10*DegToRad()) * 0.0052 / 10*DegToRad();
    else if(th <= 30*DegToRad())  NORM *= 0.9648 - (th - 20*DegToRad()) * 0.0121 / 10*DegToRad();
    else Msg(EsafMsg::Panic)<< "<ApplyDQE> Angle on pupil SHOULD NOT be so large = "<<th*RadToDeg() << MsgDispatch;
    
    
    // 2. apply QE
    Double_t eff = InterpolateQE(ph.wl);
    eff *= NORM;
    if(eff < rndm->Rndm()) return 3;
    
    // if photon is still alive
    return 0;    
}   

//______________________________________________________________________________
Double_t SimpleDetector::InterpolateQE(Double_t wl) const {
    //
    // interpolate using tabulated values
    //
    
    Double_t rtn = 0.;
    Int_t index = Int_t(floor(wl/(10.*nm)));
    index -= 20;
    if(wl <= 200*nm || wl >= 710*nm) rtn = 0.;
    else if(index < 0) Msg(EsafMsg::Warning)<< "<ApplyDQE> index SHOULD NOT be negative, (wl, index) = ("<<wl/nm<<", "<<index<<")" << MsgDispatch;
    else rtn = fPMQE[index] + (wl/(10.*nm) - floor(wl/(10.*nm))) * (fPMQE[index+1] - fPMQE[index]);

    return rtn;
}

//______________________________________________________________________________
Double_t SimpleDetector::InterpolateBG3(Double_t wl) const {
    //
    // interpolate using tabulated values
    //
    
    Double_t rtn = 0.;
    Int_t index = Int_t(floor(wl/nm));
    index -= 200;
    if(wl <= 200*nm || wl >= 600*nm) rtn = 0.;
    else if(index < 0) Msg(EsafMsg::Warning)<< "<ApplyOpticsFilter> index SHOULD NOT be negative, (wl, index) = ("<<wl/nm<<", "<<index<<")" << MsgDispatch;
    else rtn = fBG3filter[index] + (wl/nm - floor(wl/nm)) * (fBG3filter[index+1] - fBG3filter[index]);

    return rtn;
}

//______________________________________________________________________________
void SimpleDetector::SpaceTimePEinfos(const vector<PhotoElectron*>& templist) {
    //
    // - orders pe- chronologically
    //   then find the time of first GTU and ranges them in the map using GTU info
    // - Set pixel ID
    //


    // 1. find time of first and last created pe-
    fFirstPETime = HUGE;
    fLastPETime = 0.;
    TArrayD tofarray(fNbSignalPE);
    for(size_t i=0; i < fNbSignalPE; i++) {
	tofarray[i] = templist[i]->Time();
        if(tofarray[i] < fFirstPETime) fFirstPETime = tofarray[i];
        if(tofarray[i] > fLastPETime)  fLastPETime  = tofarray[i];
    }
    
    
    // 2. find time of first GTU (first pe- sampled randomly in the first GTU)
    fFirstGTUTime = fFirstPETime - fGTUlength * EsafRandom::Get()->Rndm();
    fNbGTUs = Int_t(ceil( (fLastPETime - fFirstGTUTime) / fGTUlength ));
    for(size_t i=0; i < fNbGTUs + 1; i++) fPhotoElectrons[i] = NULL;
    
    
    // 3. sort tofarray elements using time info
    Int_t sortedindex[fNbSignalPE];
    Sort(fNbSignalPE,tofarray.GetArray(),sortedindex,false); // false means in increasing order
    
    
    // 4. find pixelID for each pe-
    //    and fill fPhotoElectrons using GTU info
    Double_t tof = 0.;
    UInt_t pixeldID = 0;
    PhotoElectron* pe = 0;
    UInt_t GTUnb(0), oldGTUnb(100000);
    for(size_t i=0; i < fNbSignalPE; i++) {
        pe = templist[sortedindex[i]];
	pixeldID = FindPixelID(*pe);
	pe->SetPixelID(pixeldID);
        tof = pe->Time();
	GTUnb = UInt_t(floor( (tof - fFirstGTUTime) / fGTUlength ));
	if(GTUnb == 0 && oldGTUnb == 100000)   fPhotoElectrons[0] = new vector<PhotoElectron*>;
	else if(oldGTUnb < GTUnb)              fPhotoElectrons[GTUnb] = new vector<PhotoElectron*>;
	else if(GTUnb > 0 && oldGTUnb > GTUnb) Msg(EsafMsg::Panic)<< "<SpaceTimePEinfos> It seems pe- are not chronologically sorted "<< MsgDispatch;
	fPhotoElectrons[GTUnb]->push_back(pe);
	oldGTUnb = GTUnb;
    }
}

//______________________________________________________________________________
UInt_t SimpleDetector::FindPixelID(const PhotoElectron& pe) const {
    //
    // find pe- pixel
    // - pixel nb 0 is at left-up corner (X<0 and Y>0)
    // - using sqrt(x*x + y*y) value in the MES Z=0 plane
    // - i,j in [0..n-1]  (i<-->row    j<-->column)
    // - pixeldID = N --> i,j coordinates found back using N / n = i  and  N % n = j
    //

    // find parent photon's horizontal position in the MES plane Z=0
    // intersection between Z=0 plane and track defined by (photon pos and (EUSO - pos).Unit() direction)
    EarthVector pos = pe.AtmoPos();
    EarthVector dir = (pos - fGeometry->GetPos()).Unit();
    Double_t mag(0);
    mag = -pos.Z() / dir.Z();
    
    EarthVector posZ0 = pos + mag*dir;
    if(fabs(posZ0.Z()) > kAltitudeTolerance) Msg(EsafMsg::Warning)<< "<FindPixelID> search pos Z coo. NOT EQUAL to zero = "<<posZ0 << MsgDispatch;
    
    
    // get pixel ID
    UInt_t j = UInt_t( floor((posZ0.X() - fFirstPixelX) / fPixelSideLength) );
    UInt_t i = UInt_t( floor((fFirstPixelY - posZ0.Y()) / fPixelSideLength) );
    
    return i*fNbPixelOnSide + j;
}

//______________________________________________________________________________
Double_t SimpleDetector::MeanBgndPerPixelPerGTU(Double_t bckgnd_photon) {
    //
    // input : bckgnd_photon is in ph / m2 / ns / sr
    // Calculate mean (over wl) background in pe/pixel/GTU (at Nadir)
    // Background wl spectrum is assumed FLAT
    //
    // NB : 500 ph/ns/sr/m2 within 300-400nm           --> ~1.7 pe/pixel/GTU
    //      500 ph/ns/sr/m2 EXTENDED within 200-600nm  --> ~2.8 pe/pixel/GTU
    //
    
        
    // 0. Set bounds according to background mode used
    Double_t wlmin(0.), wlmax(0.);
    Double_t WlRangeCorrection(1.);
    if(fBgndMode == "RedBook") {
        wlmin = 300*nm;
	wlmax = 400*nm;
    }
    else if(fBgndMode == "Extended") {
        wlmin = 200*nm;
	wlmax = 600*nm;
	WlRangeCorrection = 4.;  // rough extrapolation
    }
    
    
    // 1. find wl-averaged efficiencies of QE and BG3 filters (convolution)
    Double_t wl = wlmin;
    Double_t wlincrement = 0.1*nm;
    Double_t meaneff(0.), nbsteps(0.);
    while(wl < wlmax) {
         meaneff += InterpolateQE(wl)*InterpolateBG3(wl);
	 wl += wlincrement;
	 nbsteps += 1.;
    }
    meaneff /= nbsteps;
    // effects of optics absorption + PM collection efficiency
    meaneff *= 0.67025 * 0.53;
    
    // 2. Detector surface + pixel solid angle effects + GTUlength + wl range used
    Double_t pixomega = fPixelSideLength*fPixelSideLength / fGeometry->GetPos().Z() / fGeometry->GetPos().Z();
    Double_t eusosurf = Pi() * fGeometry->GetRadius()/m * fGeometry->GetRadius()/m;
    Double_t integ_factor = pixomega * eusosurf * fGTUlength/nanosecond * WlRangeCorrection;
    
    
    return bckgnd_photon * integ_factor * meaneff;
}

//______________________________________________________________________________
Int_t SimpleDetector::AddBackground() {
    //
    // Add bgnd in pixel/GTU holding signal pe-
    //
    
    Int_t rtn(0);
    
    // loop on GTUs
    map<UInt_t, vector<PhotoElectron*>* >::const_iterator it;
    for(it = fPhotoElectrons.begin(); it != fPhotoElectrons.end(); it++) {    
	vector<PhotoElectron*>* pelist = it->second;

	// if photons exist in this GTU
	if(pelist) {
	    
	    // init
	    Int_t prev_pixID = -1;
	    Int_t pixID = 0;
            TArrayI pixelIDs(pelist->size());
	    
	    // sort pe- according to pixelID
	    for(size_t n=0; n < pelist->size(); n++) pixelIDs[n] = Int_t((*pelist)[n]->PixelID());
            Int_t sortedindex[pelist->size()];
            Sort(pelist->size(),pixelIDs.GetArray(),sortedindex,false); // false means in increasing order
	    
	    // then add background ONCE per pixel
	    // apply DQE and BG3 dependence with incident angle
	    Int_t nbPEbgnd = 0;
	    vector<PhotoElectron*> buffer;
	    for(size_t m=0; m < pelist->size(); m++) {
	        nbPEbgnd = 0;
		pixID = Int_t((*pelist)[sortedindex[m]]->PixelID());
		if(pixID > prev_pixID) {
		    Double_t NORM(1.);
		    // find pixel corresponding incident angle
		    UInt_t i = pixID / fNbPixelOnSide;
		    UInt_t j = pixID % fNbPixelOnSide;
		    EarthVector pixelpos(fFirstPixelX + (Double_t(j)+0.5)*fPixelSideLength,fFirstPixelY - (Double_t(i)+0.5)*fPixelSideLength,0);
		    Double_t th = (fGeometry->GetPos() - pixelpos).Theta();
	            // then apply optics absorption + BG3 angle dependence
		    if(th <= 10*DegToRad())       NORM = (1. - th * 0.018868 / 10*DegToRad()) * (1. - th * 0.03 / 10*DegToRad());
	            else if(th <= 20*DegToRad())  NORM = (0.98113 - (th - 10*DegToRad()) * 0.037736 / 10*DegToRad()) * (0.97 - (th - 10*DegToRad()) * 0.0052 / 10*DegToRad());
	            else if(th <= 30*DegToRad())  NORM = (0.943396 - (th - 20*DegToRad()) * 0.207547 / 10*DegToRad()) * (0.9648 - (th - 20*DegToRad()) * 0.0121 / 10*DegToRad());
	            else {
			Msg(EsafMsg::Warning)<< "<AddBackground> Background photon seems to be OUT OF FOV = "<<th*RadToDeg() << MsgDispatch;
			Msg(EsafMsg::Warning)<< "<AddBackground> Set to 30deg" << MsgDispatch;
			NORM = (0.943396 - 0.207547) * (0.9648 - 0.0121);
		    }
		    
		    // create background pe-
		    nbPEbgnd = EsafRandom::Get()->Poisson(NORM * fMeanBgnd);
		    PhotoElectron* pe = 0;
		    for(Int_t p=0; p < nbPEbgnd; p++) {
		        pe = new PhotoElectron();
			rtn++;
			pe->SetTime(it->first * fGTUlength + fFirstGTUTime + 0.5*fGTUlength);
			pe->SetPixelID(UInt_t(pixID));
			buffer.push_back(pe);
		    }
		}
		prev_pixID = pixID;
	    }
	    for(size_t m=0; m < buffer.size(); m++)  pelist->push_back(buffer[m]);
	    buffer.clear();
	}
    }
    
    return rtn;
}

//______________________________________________________________________________
Int_t SimpleDetector::ApplyTrigger() {
    //
    // kind of trigger simulation
    // Tag pe- according to the SNR per pixel per GTU : if (S + B) >= (<B> + 3*sigmaB), pe- is considered triggered
    //
	    
    Int_t rtn(0);
    
    // loop on GTUs
    map<UInt_t, vector<PhotoElectron*>* >::const_iterator it;
    for(it = fPhotoElectrons.begin(); it != fPhotoElectrons.end(); it++) {    
	vector<PhotoElectron*>* pelist = it->second;

	// if photons exist in this GTU
	if(pelist) {
	    
	    // init
	    size_t pelistsize = pelist->size();
	    Int_t prev_pixID = -1;
	    Int_t pixID = 0;
	    Int_t nbpe_inpixel = 0;
            TArrayI pixelIDs(pelistsize);
	    
	    // sort pe- according to pixelID
	    for(size_t n=0; n < pelistsize; n++) pixelIDs[n] = Int_t((*pelist)[n]->PixelID());
            Int_t sortedindex[pelistsize];
            Sort(pelistsize,pixelIDs.GetArray(),sortedindex,false); // false means in increasing order
	    
	    // loop on pe- hold in this GTU
	    // treatment pixel per pixel
            PhotoElectron* pe = 0;
	    vector<PhotoElectron*> buffer;
	    for(size_t m=0; m < pelistsize; m++) {
		pe = (*pelist)[sortedindex[m]];
		pixID = Int_t(pe->PixelID());
		if(pixID > prev_pixID) {
		    // another pixel is considered, work out SNR treatment for previous pixel
		    Double_t SNR = Float_t(nbpe_inpixel) / fMeanBgnd;
                    for(size_t t=0; t < buffer.size(); t++) {
		        buffer[t]->SetSNR(SNR);
		        rtn++;
                        if(Float_t(nbpe_inpixel) > (fMeanBgnd + fTriggerLevel*Sqrt(fMeanBgnd))) buffer[t]->Triggered();
		        else rtn--;
		    }
		    // reset for next pixel
		    buffer.clear();
		    nbpe_inpixel = 1;
		}
		else if(pixID == prev_pixID) {
		    nbpe_inpixel++;
		}
		else Msg(EsafMsg::Panic)<< "<ApplyTrigger> pixelID are not increasingly sorted : SHOULD NOT"<< MsgDispatch;
		buffer.push_back(pe);
		prev_pixID = pixID;
	    }
	    // SNR treatment for last pixel
            Double_t SNR = Float_t(nbpe_inpixel) / fMeanBgnd;
            for(size_t t=0; t < buffer.size(); t++) {
                buffer[t]->SetSNR(SNR);
                rtn++;
                if(Float_t(nbpe_inpixel) > (fMeanBgnd + fTriggerLevel*Sqrt(fMeanBgnd))) buffer[t]->Triggered();
                else rtn--;
            }
	    buffer.clear();
	}
    }
    
    return rtn;
}

