// $Id: MCRadiativeTransfer.hh,v 1.14 2006/03/20 21:51:35 thea Exp $
// Author: Sylvain Moreggia   2005/08/16

/*****************************************************************************
 * ESAF: Euso Simulation and Analysis Framework                              *
 *                                                                           *
 *  Id: MCRadiativeTransfer                                                  *
 *  Package: radiativetransfer                                               *
 *  Coordinator: Sylvain Moreggia                                            *
 *                                                                           *
 *****************************************************************************/

#ifndef __MCRADIATIVETRANSFER_HH__
#define __MCRADIATIVETRANSFER_HH__

#include "euso.hh"
#include "RadiativeTransfer.hh"
#include "DetectorGeometry.hh"
#include "EarthVector.hh"
#include "SinglePhotonPropagator.hh"

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// MCRadiativeTransfer                                                        //
//                                                                            //  
// Detector Solid Angle applied to reduce nb of photons to propagate          //  
// then a full MC simu is done                                                //  
//                                                                            //  
//  NB : aerosols not simulated in this propagation mode                      //  
//                                                                            //  
////////////////////////////////////////////////////////////////////////////////


class PhotonsInAtmosphere;
class ListPhotonsInAtmosphere;
class ListPhotonsOnPupil;
class BunchOfPhotons;
class Ground;
class ClearSkyPropagator;
class InCloudsPropagator;
class SinglePhoton;
class LowtranRadiativeProcessesCalculator;
class FastRadiativeProcessesCalculator;

class EEvent;


class MCRadiativeTransfer : public RadiativeTransfer {
public:
    MCRadiativeTransfer();
    virtual ~MCRadiativeTransfer();
    
    // Transfer photons in atmosphere, return a list of photons on Euso pupil
    virtual PhotonsOnPupil* Get(PhotonsInAtmosphere*, const DetectorGeometry* dg);
    
    // get ready for next event
    void Reset();
        
    // release all the memoty hold in the buffers
    Bool_t ClearMemory(); 

    EsafConfigClass(RadiativeTransfer,MCRadiativeTransfer)
    

private:
    // Get photons directly emitted within the Euso solid angle
    // stored in fTotalList, THEY WILL NOT UNDERGO SCATTERING SIMU
    void DirectToEuso(const BunchOfPhotons&);
    
    // - Detector solid angle is applied here -> the resulting photons will be propagated
    // - Photon features are set here (lambda, position, direction)
    // - if nb of photons is > 1e6, generation is splitted in several parts, creating 1e6 photons each time
    //   then, present method returns remaining nb of photons to be generated
    Int_t PhotonsInOmega(const BunchOfPhotons&,Int_t);
    
    // Scan fTempList before filling fTotalList, applying relevant cuts
    void ScanTempList();
        
    // Final phase of the transfer : propagation of the SinglePhotons
    // always uses LowtranCalculator, to get ozone effect
    void PropagateAllToDetector();
    
    // same as PropagateToDetector but :
    // - act on fTempList
    // - disable fine transmission study
    void PropagateTempListToDetector();
    
    // generate PhotonsOnPupil (fPhotons) from ListPhotonsInAtmosphere (fTotalList)
    void ConvertIntoPonP();
    
    // for ground detector only, assumed to be half a sphere
    // if dist(photon,detec) < fDistance_cut, photon is lost 
    // distribute photons on a sphere : if photons are on the bottom hemisphere, they are lost
    Bool_t IsSeenByGroundDetector(const SinglePhoton&) const;
    
    ListPhotonsOnPupil* fPhotons;         // keep in memory the list returned to detector
    ListPhotonsInAtmosphere* fTotalList;  // pointer on the list created by LightSource module
					  // STORE SINGLEPHOTONS AFTER THEIR SCATTERING SIMU
                                          // once inside a photon MUST NOT BE DELETED before conversion into ParentPhoton
					  // _must not be reset nor deleted_ by the present class

    SinglePhotonPropagator* fPropagator;               // propagator for scattering simulation
    LowtranRadiativeProcessesCalculator* fLowToDetec;  // calculator for last transmission to detector
    vector<SinglePhoton*> fTempList;                   // photons kept here during scattering simu. Then copied in fTotalList

    
    size_t fNbBunch;                      // number of bunches to handle
    Double_t fNbTot;                      // total number of photons created by LightSource
    Double_t fNbSingles;                  // total nb of photons to propagate
    Double_t fMaxPhNb;                    // max nb of photons to be transported --> if nb of photons is larger, simu is splitted in several steps
    Int_t fScatOrder;                     // scattering order to be simulated
    Double_t fMaxPhaseFunction;           // to normalize all the phase function
    Double_t fOmegaMax;                   // factor which determines nb of photons to be simulated
    Int_t fMScattAnalysis;                // if >0, scattering simu is switched to pure MScatt analysis
                                          // 1 : check if photons goes out atmosphere, 2 : does not check

    Double_t fDistance_cut;               // for ground detector mode : if dist(photon,detec) < fDistance_cut, photon is lost
    Bool_t fKeepAll;                      // tell if simu keeps also photons not transimitted to detector
                                          // ALLOW to fine study transmission values along last travel to detector
					  // if disabled : it reduces memory compsumption (particularly for GROUND detector case)

    
    // copy of detector geometry defined in Detector part 
    inline virtual void CopyDetectorGeometry(const DetectorGeometry* dg);

    ClassDef(MCRadiativeTransfer,0)
};

inline void MCRadiativeTransfer::CopyDetectorGeometry(const DetectorGeometry* dg) {
    //
    // set DetectorGeometry for the propagator
    //
    fDetGeom = dg;
    fPropagator->CopyDetectorGeometry(fDetGeom,fDecoupled);
}

#endif  /* __MCRADIATIVETRANSFER_HH__ */

